#!/usr/bin/env python
"""
Test module to check segclassmodel
"""
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.segment3d.segclassmodel import SegClassModelPar, SegClassModel
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentclass import HelixClassPreparation
from spring.tests.segment3d.test_segclassreconstruct import TestSegClassReconstructPreparation
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dClean, TestSegmentRefine3dPreparationDatabase
import numpy as np
import os
import random


class TestSegClassModelPreparation(object):
    def generate_series_of_volumes(self):
        segment_size = int(self.volume_size_in_angstrom/self.pixelsize)
        class_stack = []
        prj_params = []
        for each_rise_rotation, each_radius, each_model in zip(self.symmetries, self.helix_radii_in_angstrom, self.models):
            helical_rise, helical_rotation = each_rise_rotation
            complete_euler = SegmentRefine3d().generate_Euler_angles_for_projection(self.azimuthal_count,
            self.out_of_plane_range, self.out_of_plane_count, helical_rotation)
        
            helix_volume =\
            TestSegClassReconstructPreparation().generate_helical_volume_square(self.size_of_subunit_in_angstrom,
            self.pixelsize, segment_size, each_radius, helical_rise, helical_rotation)
            
            helix_volume = SegClassReconstruct().set_header_with_helical_parameters(each_rise_rotation, helix_volume,
            'c{0}'.format(self.helix_start))
            helix_volume.write_image(each_model)
            
            random_prj_params = [complete_euler[random.randint(0, len(complete_euler) - 1)] for each in list(range(3))]
            prj_params.append(random_prj_params)
            class_stack += TestSegClassReconstructPreparation().generate_noise_projections_from_helix(None,
            helix_volume, random_prj_params)

        coord = np.arange(len(prj_params) * 10 * self.pixelsize)
        coordinates = ((coord, coord))
        TestSegmentRefine3dPreparationDatabase().prepare_database_for_refinement(np.vstack(prj_params), self.pixelsize,
        coordinates)

        for each_id, each_class in enumerate(class_stack):
            each_class.write_image(self.test_averages, each_id)
        
        
class TestSegClassModelSetup(TestSegClassModelPreparation):
    def setup_main_segclassmodel_params(self):
        self.size_of_subunit_in_angstrom = 50
        self.helix_radii_in_angstrom = [25, 50, 50]
        self.width_of_helix_in_angstrom = max(self.helix_radii_in_angstrom)
        self.symmetries = [(8, 40), (10, 50), (12, 60)]
        self.models = ['model{0:03}.hdf'.format(each_model) for each_model in list(range(3))]
        self.volume_size_in_angstrom = 400
        
        self.test_averages = 'test_averages.hdf'
        self.x_translation = 50
        self.y_translation = 10
        self.azimuthal_count = 3
        self.out_of_plane_range = (-8, 8)
        self.out_of_plane_count = 3
        self.helix_start = 1
        self.pixelsize = 10.0


    def setup_common_non_mpi_parameters(self):
        self.custom_filter_file = 'test_filter_file.dat'
        self.test_bin_factor = 2
        HelixClassPreparation().prepare_custom_filter_file(self.custom_filter_file, self.test_bin_factor)
        self.generate_series_of_volumes()

        self.feature_set.parameters['Class average stack'] = self.test_averages
        self.feature_set.parameters['Reference volumes']='model00?.hdf'
        
        self.feature_set.parameters['spring.db file']=os.path.join(os.pardir, 'spring.db')
        
        self.feature_set.parameters['Classes select option']=False
        self.feature_set.parameters['Include or exclude classes']='exclude'           
        self.feature_set.parameters['Classes list']='3, 4' 

        self.feature_set.parameters['Limit in-plane rotation']=True
        self.feature_set.parameters['Delta in-plane rotation angle']=10.0
        self.feature_set.parameters['X and Y translation range in Angstrom']=((self.x_translation, self.y_translation))
        self.feature_set.parameters['Out-of-plane tilt angle range']=self.out_of_plane_range

        self.feature_set.parameters['Number of projections azimuthal/out-of-plane angle']=((self.azimuthal_count,
        self.out_of_plane_count))

        self.feature_set.parameters['Keep intermediate files']=False

        self.feature_set.parameters['Pixel size in Angstrom'] = float(self.pixelsize)
        self.feature_set.parameters['Estimated helix width and height in Angstrom'] = ((self.width_of_helix_in_angstrom,
        int(1.5 * self.width_of_helix_in_angstrom)))

        self.feature_set.parameters['Internal binning factor'] = self.test_bin_factor

        self.feature_set.parameters['Limit in-plane rotation'] = True
        self.feature_set.parameters['Delta in-plane rotation angle'] = 10.0
        self.feature_set.parameters['X and Y translation range in Angstrom'] = (20, 15)
        self.feature_set.parameters['High-pass filter option'] = False
        self.feature_set.parameters['Low-pass filter option'] = False
        self.feature_set.parameters['High and low-pass filter cutoffs in 1/Angstrom'] = (0.01, 0.06)
        self.feature_set.parameters['Custom filter option'] = False
        self.feature_set.parameters['Custom-built filter file'] = self.custom_filter_file
        self.feature_set.parameters['B-Factor']=0

        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=1
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)
            

class TestSegClassModel(TestSegClassModelSetup, SegClassModel):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = SegClassModelPar()
        self.setup_main_segclassmodel_params()
        self.setup_common_non_mpi_parameters()
        
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())

        super(TestSegClassModel, self).__init__(self.feature_set)


    def teardown(self):
        os.remove(self.custom_filter_file)
        os.remove(self.test_averages)
        [os.remove(each_file) for each_file in self.models if os.path.exists(each_file)]
        os.remove('spring.db')
        os.remove(os.path.join(os.pardir, 'spring.db'))
        
        self.testingdir.remove()


class TestSegClassModelMain(TestSegClassModel):

    def do_test_case_scm1(self):
        """
        * Standard single-CPU segclassmodel test
        """
        self.match_reprojections_to_classes()
        TestSegmentRefine3dClean().prepare_final_prj_for_documentation(self.models)


class TestSegClassModelMore(TestSegClassModel):
    def do_test_case_scm2(self):
        """
        * Single-CPU segclassmodel test including class selection
        """
        self.feature_set.parameters['Classes select option']=True
        super(TestSegClassModel, self).__init__(self.feature_set)
        self.match_reprojections_to_classes()


class TestSegClassModelEndToEnd(TestSegClassModel):
    def do_end_to_end_test_scm_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_scm_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)


class TestSegClassModelMpi(TestSegClassModel):
    def do_end_to_end_test_scm_inputfile_MPI(self):

        self.feature_set.parameters['MPI option']=True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)


def main():
    tsce = TestSegClassModelMain()
    tsce.setup()
    tsce.do_test_case_scm1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 21-Sep-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment2d.segmentctfapply import SegmentCtfApply, SegmentCtfApplyPar
import os
import shutil

        
class SegmentCtfApplyMpi(SegmentCtfApply):
    def apply_ctf_to_segment_stack(self):
        
        self.comm, self.rank, self.size, self.log, self.tempdir = OpenMpi().setup_mpi_and_simultaneous_logging(self.log,
        self.feature_set.logfile, self.temppath)
        
        if self.rank == 0:
            shutil.copy(self.spring_path, 'spring.db')
            
            ctf_parameters = \
            self.get_ctf_values_from_database_and_compute_local_ctf_based_if_demanded(self.ctffind_or_ctftilt_choice,
            self.convolve_or_phaseflip_choice, self.astigmatism_option, self.pixelsize, self.spring_path)
            
            segment_ids = list(range(len(ctf_parameters)))
            
            ctf_parameters = OpenMpi().split_sequence_evenly(ctf_parameters, self.size)
            segment_ids = OpenMpi().split_sequence_evenly(segment_ids, self.size)
        else:
            ctf_parameters = None
            segment_ids = None
        
        ctf_parameters = self.comm.scatter(ctf_parameters, root=0)
        segment_ids = self.comm.scatter(segment_ids, root=0)
        
        local_outfile = self.tempdir + self.outfile
        
        self.apply_ctf_to_segments(segment_ids, ctf_parameters, self.convolve_or_phaseflip_choice, self.infile,
        local_outfile)
        
        self.comm.barrier()
        OpenMpi().gather_stacks_from_cpus_to_common_stack(self.comm, local_outfile, self.outfile)
        
        os.rmdir(self.tempdir)
        if self.rank == 0:
            self.log.endlog(self.feature_set)
        

def main():
    parset = SegmentCtfApplyPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    segment_stack = SegmentCtfApplyMpi(reduced_parset)
    segment_stack.apply_ctf_to_segment_stack()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segmentselect
"""
from EMAN2 import EMUtil
from spring.csinfrastr.csdatabase import SpringDataBase, base, SegmentTable
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment2d.segmentselect import SegmentSelectPar, SegmentSelect
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dSetupForOthers
import os
import sys


class TestSegmentSelectPreparation(object):
    def setup_segmentrefine3d(self):
        self.tsr3d = TestSegmentRefine3dSetupForOthers()
        self.tsr3d.setup_sr3d()


    def check_filtered_values_in_or_outside_range(self, each_value, value_range, in_or_exclude):
        if in_or_exclude == 'include':
            sys.stderr.write('\n:{0} <= {1} <= {2}'.format(min(value_range), each_value, max(value_range)))
            assert min(value_range) <= each_value <= max(value_range)
        if in_or_exclude == 'exclude':
            sys.stderr.write('\n{1} < {0}\n{1} > {2}'.format(min(value_range), each_value, max(value_range)))
            assert each_value < min(value_range) or each_value > max(value_range)


    def check_values_in_filtered_database(self):
        self.session = SpringDataBase().setup_sqlite_db(base, self.test_database)
        segments = self.session.query(SegmentTable).all()
        
        for each_segment in segments:
            defocus_range = ((self.feature_set.parameters['Defocus range']))
            defocus_in_or_exclude = self.feature_set.parameters['Include or exclude defocus range']
            
            self.check_filtered_values_in_or_outside_range(each_segment.avg_defocus, defocus_range,
            defocus_in_or_exclude)
            
            sys.stderr.write('\nSelected defoci checked. ok')
            
            astigmatism_range = ((self.feature_set.parameters['Astigmatism range']))
            astigmatism_in_or_exclude = self.feature_set.parameters['Include or exclude astigmatic segments']
            
            self.check_filtered_values_in_or_outside_range(each_segment.astigmatism, astigmatism_range,
            astigmatism_in_or_exclude)
            
            sys.stderr.write('\nSelected astigmatism checked. ok')
    
#             curvature_range = ((self.feature_set.parameters['Persistence length range']))
#             curvature_in_or_exclude = self.feature_set.parameters['Include or exclude straight helices']
#             
#             self.check_filtered_values_in_or_outside_range(each_segment.curvature, curvature_range,
#             curvature_in_or_exclude)
#             
#             sys.stderr.write('\nSelected straightness checked. ok')
#             
#             layer_cc_range = ((self.feature_set.parameters['Correlation layer line range']))
#             
#             layer_cc_in_or_exclude = self.feature_set.parameters['Include or exclude segments based on layer-line ' + \
#             'correlation']
#             
#             self.check_filtered_values_in_or_outside_range(each_segment.ccc_layer, layer_cc_range,
#             layer_cc_in_or_exclude)
#             
#             sys.stderr.write('\nSelected layer-line correlation checked. ok')
            
    def check_number_of_images_on_output_stack(self):
        assert EMUtil.get_image_count(self.test_selected_stack) == self.session.query(SegmentTable).count()
        
    def check_results(self):
        self.check_values_in_filtered_database()
        self.check_number_of_images_on_output_stack()
        
    
    def prepare_segment_file(self):
        sfile = open(self.test_segment_file, 'w')
        
        stack_ids = ['{0}\n'.format(each_stack_id) for each_stack_id in list(range(5)) + list(range(10, 55))]
        sfile.writelines(stack_ids)
        sfile.close()

        
    def setup_selection_criteria_from_segment_table(self, feature_set):
        feature_set.parameters['Segments select option'] = True
        feature_set.parameters['Include or exclude segments'] = 'include'
        feature_set.parameters['Segment file'] = self.test_segment_file
        
        feature_set.parameters['Micrographs select option'] = True
        feature_set.parameters['Include or exclude micrographs'] = 'include'
        feature_set.parameters['Micrographs list'] = '1'
        
        feature_set.parameters['Helices select option'] = True
        feature_set.parameters['Include or exclude helices'] = 'include'
        feature_set.parameters['Helices list'] = '1,2'
        
        feature_set.parameters['Classes select option'] = True
        feature_set.parameters['Include or exclude classes'] = 'include'
        feature_set.parameters['Class type']='class_id'
        feature_set.parameters['Classes list'] = '0, 1, 2'
        
        feature_set.parameters['Persistence class option'] = True
        feature_set.parameters['Persistence class length in Angstrom'] = 700
        feature_set.parameters['Class occupancy threshold']=0.5

        feature_set.parameters['Straightness select option'] = True
        feature_set.parameters['Include or exclude straight helices'] = 'include'
        feature_set.parameters['Persistence length range'] = (0, 80)
        
        feature_set.parameters['Defocus select option'] = True
        feature_set.parameters['Include or exclude defocus range'] = 'include'
        feature_set.parameters['Defocus range'] = (0, 23000)
        
        feature_set.parameters['Astigmatism select option'] = True
        feature_set.parameters['Include or exclude astigmatic segments'] = 'include'
        feature_set.parameters['Astigmatism range'] = (0, 10000)
        
        feature_set.parameters['Layer line correlation select option'] = True
        feature_set.parameters['Include or exclude segments based on layer-line correlation'] = 'include'
        feature_set.parameters['Correlation layer line range']=((0, 80))
        
        return feature_set
    
    
class TestSegmentSelect(TestSegmentSelectPreparation, SegmentSelect, SegmentSelectPar):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.setup_segmentrefine3d()
        self.feature_set = SegmentSelectPar()
        
        self.test_stack = self.tsr3d.input_stack
        self.test_selected_stack = 'test_selected.hdf'
        self.test_database = 'test_selected.db'
        self.test_segment_file = 'test_segment_file.dat'

        self.prepare_segment_file()
        self.feature_set.parameters['Image input stack']=self.test_stack
        self.feature_set.parameters['Image output stack']=self.test_selected_stack
        
        self.feature_set.parameters['spring.db file']=os.path.join(os.pardir, 'spring.db')
        self.feature_set.parameters['Selected database']=self.test_database
            
        self.feature_set = self.setup_selection_criteria_from_segment_table(self.feature_set)

        super(TestSegmentSelect, self).__init__(self.feature_set)
        

    def teardown(self):
        database_file = 'spring.db'
        os.remove(database_file)
        os.remove(os.path.join(os.pardir, database_file))
        os.remove(self.test_database)
        os.remove(self.test_segment_file)
        
        os.remove(self.test_stack)
        os.remove(self.test_selected_stack)

        self.testingdir.remove()


class TestSegmentSelectMain(TestSegmentSelect):
    def do_test_case_scex1(self):
        self.extract_segments()
        self.check_results()


# class TestSegmentSelectEndToEnd(TestSegmentSelect):
#     def do_end_to_end_test_scex_inputfile(self):
#         EndToEndTest().do_end_to_end_inputfile(self.feature_set)
#         self.check_results()
#         
# 
#     def do_end_to_end_test_scex_prompt(self):
#         EndToEndTest().do_end_to_end_prompt(self.feature_set)
#         self.check_results()
        

class TestSegmentSelectMore(TestSegmentSelect):
    def do_test_case_scex2(self):
        self.feature_set.parameters['Include or exclude defocus range'] = 'exclude'
        self.feature_set.parameters['Defocus range'] = (0, 12000)
        self.feature_set.parameters['Include or exclude segments based on layer-line correlation'] = 'exclude'
        self.feature_set.parameters['Correlation layer line range']=((0.1, 0.4))
        
        self.feature_set.parameters['Include or exclude straight helices'] = 'exclude'
        self.feature_set.parameters['Include or exclude classes'] = 'exclude'
        self.feature_set.parameters['Class type']='class_model_id'
        self.feature_set.parameters['Classes list'] = '0'
        
        super(TestSegmentSelect, self).__init__(self.feature_set)
        self.extract_segments()
        self.check_results()
    
def main():
    tscex = TestSegmentSelectMain()
    tscex.setup()
    tscex.do_test_case_scex1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import Util, EMData, EMUtil, EMNumPy
from collections import namedtuple
from filter import filt_gaussl
from fundamentals import rot_shift2D, image_decimate, fshift
from morphology import threshold
from scipy import interpolate
from spring.csinfrastr.csdatabase import SpringDataBase, base, CtfMicrographTable, HelixTable, SegmentTable, \
    CtfFindMicrographTable, CtfTiltMicrographTable, refine_base, RefinementCycleSegmentTable, RefinementCycleTable, \
    RefinementCycleHelixTable
from spring.csinfrastr.csreadinput import OptHandler
from spring.micprgs.scansplit import Micrograph
from spring.segment2d.segment_int import SegmentStraighten
from spring.segment2d.segment_prep import SegmentPar
from spring.segment2d.segmentctfapply import SegmentCtfApply
from spring.segment2d.segmentexam import SegmentExam
from spring.segment2d.segmentselect import SegmentSelect
from tabulate import tabulate
import numpy as np
import os
import shutil
    
                        
class SegmentCut(SegmentStraighten):
    def compute_new_micsize_if_coordinates_at_edges(self, xcoord, ycoord, segsizepix, mic_xsize, mic_ysize):
        """
        >>> from spring.segment2d.segment import Segment
        >>> s = Segment()
        >>> s.compute_new_micsize_if_coordinates_at_edges(10, 10, 30, 2000, 4000)
        (2012, 4012, -990, -1990)
        >>> s.compute_new_micsize_if_coordinates_at_edges(1995, 3995, 30, 2000, 4000)
        (2022, 4022, 995, 1995)
        >>> s.compute_new_micsize_if_coordinates_at_edges(-10, 10, 30, 2000, 4000)
        (2052, 4012, -1010, -1990)
        >>> s.compute_new_micsize_if_coordinates_at_edges(2005, 4005, 30, 2000, 4000)
        (2042, 4042, 1005, 2005)
        >>> s.compute_new_micsize_if_coordinates_at_edges(13463, 1924, 1216, 10880, 14035)
        (17264, 14035, 8023, -5093)
        """
        x_distance_from_edge = int(min((xcoord, mic_xsize - xcoord)))
        y_distance_from_edge = int(min((ycoord, mic_ysize - ycoord)))
        xoffcenter = int(round(xcoord)) - mic_xsize / 2
        yoffcenter = int(round(ycoord)) - mic_ysize / 2
        if x_distance_from_edge <= segsizepix / 2 or y_distance_from_edge <= segsizepix / 2:
            if x_distance_from_edge <= segsizepix / 2:
                mic_xsize = int(mic_xsize + (segsizepix - 2 * (x_distance_from_edge - 1)))
            if y_distance_from_edge <= segsizepix / 2:
                mic_ysize = int(mic_ysize + (segsizepix - 2 * (y_distance_from_edge - 1)))
                
        return mic_xsize, mic_ysize, xoffcenter, yoffcenter
    

    def window_segment(self, micrograph, xcoord, ycoord, segsizepix):
        """
        * Function to window segment from image part of cut_segments

        #. Input: x and y coordinates, segment size (pixel)
        #. Output: windowed segment
        #. Usage: im_out = window_segment(micrograph_img, xcoord, ycoord, segsizepix)
        
        """

        mic_xsize = micrograph.get_xsize()
        mic_ysize = micrograph.get_ysize()
        
        new_mic_xsize, new_mic_ysize, xoffcenter, yoffcenter = self.compute_new_micsize_if_coordinates_at_edges(xcoord,
        ycoord, segsizepix, mic_xsize, mic_ysize)
        
        if new_mic_xsize > mic_xsize or new_mic_ysize > mic_ysize:
            micrograph = Util.pad(micrograph, new_mic_xsize, new_mic_ysize, 1, 0, 0, 0, 'average')
        im_out = Util.window(micrograph, segsizepix, segsizepix, 1, xoffcenter, yoffcenter, 0)

        return im_out
    

    def invert(self, img=None):
        """
        * Function to invert segment image densities for cryo data (protein=white)

        #. Input: segment image
        #. Output: inverted image segment
        #. Usage: im_out = normalize(img)
        
        """
        if img is None: 
            img = self.im_out

        im_out = img*(-1)
        return im_out

    def normalize(self, img=None):
        """
        * Function to normalize segment to image densities of average 0 and \
        standard deviation of 1 and truncate dark spots

        #. Input: segment image
        #. Output: normalized image segment
        #. Usage: im_out = normalize(img)
        
        """
        if img is None: 
            img = self.im_out

        img.process_inplace('normalize')
        # clear dark spots to average
        im_out = threshold(img, -3.75)

        return im_out

    def verticalize_segment(self, im_out=None, ipangle=None):
        """
        * Function to align helix axis to perpendicular image rows, includes additional windowing\
            to get rid of rotation artefacts at edges

        #. Input: image, angle to be rotated
        #. Output: written rtstack file
        #. Usage: verticalize_segment(im_out, ipangle)
        """
        if im_out is None: 
            im_out = self.im_out
        if ipangle is None: 
            ipangle = self.ipangle
        
        im_outrt = rot_shift2D(im_out, ipangle, 0, 0)#, interpolation_method='gridding')

        im_outrt = Util.window(im_outrt, self.segsizepix, self.segsizepix, 1, 0, 0, 0)
        
        return im_outrt
    
                
    def minimize_xposition(self, rowsaddimg=None, xshift=None):
        """
        * Function to center helix axis due to 2fold symmetry of projection

        #. Input: 1D helix projection
        #. Output: image shift required for centering perpendicular to helix axis
        #. Usage: centerx = minimize_xposition(rowsaddimg)
        
        """
        if rowsaddimg is None: 
            rowsaddimg = self.rowsaddimg

        # assume a rough centering cut out 1/6 at either sides
        symimgpars = []
        xshifts = np.arange(-xshift, xshift, 0.3)
#        pixels = np.arange(0, rowsaddimg.get_xsize())
        for shift in xshifts:
            # shift array
            shiftedline = fshift(rowsaddimg, shift)
            arrrowsaddimg = np.copy(EMNumPy.em2numpy(shiftedline))
            
            sp = np.fft.fft(arrrowsaddimg)
       
            # return a flattened array
            sp = sp.ravel()
       
            # compute symimgpar
            i1 = sp.imag**2
            i2 = sp.imag**2 + sp.real**2
            symimgpar = np.sqrt(i1.sum())/np.sqrt(i2.sum())

            symimgpars.append(symimgpar)

        # increase accuracy by a factor of 50 by interpolation
        xshifts_fine = np.arange(-xshift, xshift, 0.01)
        t = interpolate.splrep(xshifts, symimgpars, k=3, s=0)
        symimg_interpolated = interpolate.splev(xshifts_fine, t)

        sorted_indices, minima = SegmentExam().find_local_extrema(symimg_interpolated, target='minima')
        # sort according to real for display purposes
        centerx = xshifts_fine[sorted_indices[0]]
        symimg_par = symimg_interpolated[sorted_indices[0]]

        return centerx, symimg_par
    

    def center_xycoordinates(self, xcoord=None, ycoord=None, centerx=None, ipangle=None):
        """
        * Function to apply shift perpedicular to helix to xy coordinates

        #. Input: x and y coordinates, x-shift (pixel), in-plane rotation angle
        #. Output: centered x and y coordinates
        #. Usage: newxcoord, newycoord = center_xycoordinates(xcoor, ycoord, centerx, ipangle)

        >>> import numpy as np
        >>> from spring.segment2d.segment import Segment
        >>> # helix oriented along image columns
        ...
        >>> s = Segment()
        >>> x, y = s.center_xycoordinates(np.zeros(10), np.arange(10), 1, 0)
        >>> x 
        array([-1., -1., -1., -1., -1., -1., -1., -1., -1., -1.])
        >>> y
        array([ 0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9.])
        >>> # helix oriented along image rows
        ...
        >>> x = np.arange(10)
        >>> y = np.zeros(10)
        >>> s_xcoord, s_ycoord = s.center_xycoordinates(x, y, 1, 90)
        >>> s_xcoord
        array([ -6.12323400e-17,   1.00000000e+00,   2.00000000e+00,
                 3.00000000e+00,   4.00000000e+00,   5.00000000e+00,
                 6.00000000e+00,   7.00000000e+00,   8.00000000e+00,
                 9.00000000e+00])
        >>> s_ycoord
        array([-1., -1., -1., -1., -1., -1., -1., -1., -1., -1.])
        >>> # helix oriented by 45 degrees
        ...
        >>> y = x
        >>> s_xcoord, s_ycoord = s.center_xycoordinates(x, y, 1, -45)
        >>> s_xcoord
        array([-0.70710678,  0.29289322,  1.29289322,  2.29289322,  3.29289322,
                4.29289322,  5.29289322,  6.29289322,  7.29289322,  8.29289322])
        >>> s_ycoord
        array([ 0.70710678,  1.70710678,  2.70710678,  3.70710678,  4.70710678,
                5.70710678,  6.70710678,  7.70710678,  8.70710678,  9.70710678])
        >>> # helix oriented by -45 degrees
        ...
        >>> s_xcoord, s_ycoord = s.center_xycoordinates(x, np.flipud(x), 1, 45)
        >>> s_xcoord
        array([-0.70710678,  0.29289322,  1.29289322,  2.29289322,  3.29289322,
                4.29289322,  5.29289322,  6.29289322,  7.29289322,  8.29289322])
        >>> s_ycoord
        array([ 8.29289322,  7.29289322,  6.29289322,  5.29289322,  4.29289322,
                3.29289322,  2.29289322,  1.29289322,  0.29289322, -0.70710678])

        """
        if xcoord is None: xcoord = self.xcoord
        if ycoord is None: ycoord = self.ycoord
        if centerx is None: centerx = self.centerx
        if ipangle is None: ipangle = self.ipangle

        rotx = centerx*np.cos(np.radians(ipangle))
        roty = centerx*np.sin(np.radians(ipangle))
#        self.log.dlog('modified x: {0} modified y: {1} inplane-angle: {2}'.format(rotx, roty, ipangle))
        self.newxcoord = xcoord - rotx
        self.newycoord = ycoord - roty

        return self.newxcoord, self.newycoord


#     def center_segments(self, helices, rtimgstack, mask_width):
#         """
#         * Function to center helix axis 
# 
#         #. Input: in-plane rotated image stack
#         #. Output: re-cut segment stack
#         #. Usage: center_segments(rtimgstack, mask_width)
#         
#         """
#         self.log.fcttolog()
#         each_image_index = 0
#         segment = EMData()
# 
#         log_info = []
#         for each_helix_id, each_helix in enumerate(helices):
#             for each_seg_id, each_helix_coordinate in enumerate(each_helix.coordinates):
#                 if (self.stepsize == 0 and each_seg_id == 0) or \
#                 (self.stepsize == 0 and each_seg_id == len(each_helix_coordinate) - 1):
#                     pass
#                 else:
#                     segment.read_image(rtimgstack, each_image_index)
#                     filter_radius = self.pixelsize/float(max(self.helixwidth/4, 40.0))
#                     segment = filt_gaussl(segment, filter_radius)
#                     
#                     self.helixmask = SegmentExam().make_smooth_rectangular_mask(mask_width, self.segsizepix * 0.8,
#                     self.segsizepix)
#                     
#                     segment *= self.helixmask
#                     rowsaddimg = SegmentExam().project_helix(segment)
#                     centerx, symimg_par = self.minimize_xposition(rowsaddimg, 0.35*self.helixwidthpix)
#                     
#                     xcoord, ycoord = each_helix_coordinate
#                     
#                     centered_x, centered_y = self.center_xycoordinates(xcoord, ycoord, centerx,
#                     each_helix.inplane_angle[each_seg_id])
#                     
#                     each_helix.coordinates[each_seg_id] = (centered_x, centered_y)
#                     each_image_index = each_image_index + 1
#                     
#                     log_info += [[each_seg_id, each_helix.helix_id, centered_x, symimg_par, xcoord, ycoord,
#                     centered_x, centered_y, each_helix.picked_coordinates[each_seg_id][0],
#                     each_helix.picked_coordinates[each_seg_id][1]]]
# 
#             xs, ys = list(zip(*each_helix.coordinates))
#             each_helix_x_coordinates = np.array(xs)
#             each_helix_y_coordinates = np.array(ys)
#             
#             self.write_boxfile(each_helix_x_coordinates, each_helix_y_coordinates, self.segsizepix,
#             filename=each_helix.segment_list)
#             
#             helices[each_helix_id] = each_helix._replace(coordinates=zip(each_helix_x_coordinates,
#             each_helix_y_coordinates))
# 
#         header = ['segment_id', 'helix_id', 'center_x', 'minimizer', 'previous_xy', 'centered_xy', 'picked_xy']
#         msg = tabulate(log_info, header)
#         self.log.ilog('The following segments were centered:\n{0}'.format(msg))
#         
#         return helices


    def make_segment_info_named_tuple(self):
        segment_info = namedtuple('segment_info', 'mic_id x_coordinate_A y_coordinate_A')
        return segment_info

    
    def pre_loop_setup(self):
        each_image_index = 0
        log_info = []
        ctf_info = []
        micrograph_img = EMData()
        matched_mic_find = None
        micrograph_name = ''

        return micrograph_img, matched_mic_find, micrograph_name, log_info, ctf_info, each_image_index


    def get_mic_slice_from_mrc_stack(self, micrograph_name):
        vol = EMData()
        prefix, no_ending = micrograph_name.split('@')
        stack_no, ext = os.path.splitext(no_ending)
        stack_name = prefix + ext
        vol.read_image(stack_name)
        micrograph_img = Util.window(vol, vol.get_xsize(), vol.get_ysize(), 1, 0, 0, -(vol.get_zsize() / 2) + int(stack_no))

        return micrograph_img


    def read_micrograph_and_get_ctf_info(self, micrograph_img, matched_mic_find, micrograph_name, session, each_helix):
        if micrograph_name != each_helix.micrograph:
            micrograph_name = each_helix.micrograph
            if '@' not in micrograph_name:
                micrograph_img, mic_xsize, mic_ysize = Micrograph().readmic(micrograph_name)
            elif '@' in micrograph_name:
                micrograph_img = self.get_mic_slice_from_mrc_stack(micrograph_name)

            if self.ctfcorrect_option:
                if not self.spring_db_option:
                    msg = 'CTF correction requested without specifying a spring database file. Please run ' + \
                    'micctfdetermine to generate spring.db file that contains the relevant CTF information.'
                    raise ValueError(msg)
                else:
                    matched_mic_find = SegmentCtfApply().get_micrograph_from_database_by_micname(session,
                    micrograph_name, self.spring_path)
            if self.row_normalization:
                for each_row in list(range(micrograph_img.get_xsize())):
                    row = micrograph_img.get_row(each_row)
                    row.process_inplace('normalize')
                    micrograph_img.set_row(row, each_row)
            
        return micrograph_img, matched_mic_find, micrograph_name
    

    def window_segment_and_invert_normalize_and_ctf_correct(self, session, large_segsizepix, each_helix, micrograph_img,
    matched_mic_find, xcoord, ycoord, ctf_info, each_image_index):
        windowed_segment = self.window_segment(micrograph_img, xcoord, ycoord, large_segsizepix)
        if self.invertoption == True:
            windowed_segment = self.invert(windowed_segment)
        if self.normoption == True:
            windowed_segment = self.normalize(windowed_segment)
        if self.ctfcorrect_option:
            segment_info = self.make_segment_info_named_tuple()
            each_segment = segment_info._make([matched_mic_find.id, xcoord * self.pixelsize, ycoord * self.pixelsize])
            
            ctf_params, avg_defocus, astigmatism, astig_angle = \
            SegmentCtfApply().get_ctfparameters_from_database(self.ctffind_or_ctftilt_choice, self.astigmatism_option,
            self.pixelsize, session, each_segment, matched_mic_find, self.spring_path)
            
            windowed_segment = \
            SegmentCtfApply().filter_image_by_ctf_convolve_or_phaseflip(self.convolve_or_phaseflip_choice,
            windowed_segment, ctf_params)
            
            ctf_info += [[each_image_index, each_helix.helix_id, matched_mic_find.micrograph_name] + \
            [each_ctf for each_ctf in ctf_params]]
            
        return windowed_segment, ctf_info


    def rotate_and_unbend_segment_if_required(self, imgstack, each_image_index, large_segsizepix,
    each_helix, each_segment_index, coordpair, windowed_segment):
        inplane_angle = each_helix.inplane_angle[each_segment_index]
        if not self.unbending:
            segment_rotated = self.verticalize_segment(windowed_segment, inplane_angle)
            polyfit = None
        else:
            cut_coord_pair = round(coordpair[0]), round(coordpair[1])
            if each_segment_index * self.stepsize < 0.8 * float(self.window_size) or \
            (len(each_helix.coordinates) - each_segment_index) * self.stepsize < 0.8 * float(self.window_size):
                straightened_segment, polyfit, central_ip_angle = \
                self.unbend_segment_using_coordinates(windowed_segment, large_segsizepix, each_helix.coordinates,
                coordpair, cut_coord_pair, inplane_angle, 0.0)
            else:
                straightened_segment, polyfit, central_ip_angle = \
                self.unbend_segment_using_coordinates(windowed_segment, large_segsizepix, each_helix.coordinates,
                coordpair, cut_coord_pair, inplane_angle)
            segment_rotated = Util.window(straightened_segment, self.segsizepix, self.segsizepix, 1, 0, 0, 0)
            
        if self.unbending or self.rotoption:
            segment_rotated.write_image(imgstack, each_image_index)
        else:
            windowed_segment = Util.window(windowed_segment, self.segsizepix, self.segsizepix, 1, 0, 0, 0)
            windowed_segment.write_image(imgstack, each_image_index)

        return polyfit


    def get_ctfinfo_header(self):
        ctf_info = ['Local_id', 'helix_id', 'micrograph', 'avg_defocus(microm)', 'Cs(mm)', 'voltage(kV)', 'pixelsize', 
        'bfactor', 'amp_contrast', 'astigmatism', 'astig_angle']
        
        return ctf_info
    

    def get_log_info_header(self):
        return ['Local_id', 'helix_id', 'micrograph', 'x_coordinate', 'y_coordinate', 'normalization', 
            'contrast_inversion', 'rotation', 'unbending', 'polynomial']


    def cut_segments(self, helices=None, imgstack=None):
        """
        * Function to cut segments from helices dictionary including micrograph\
         information, 

        #. Input: helix dictionary containing: micrograph, segment_list, \
            directory, coordinates, in-plane angle; outfilestack
        #. Output: written stack
        #. Usage: cut_segments(helices, outfile)
        
        """
        self.log.fcttolog()

        session = None
        if self.spring_db_option:
            session = SpringDataBase().setup_sqlite_db(base, self.spring_path)
        if self.spring_db_option and self.frame_option:
            session = SpringDataBase().setup_sqlite_db(base, self.spring_db_frames)

        large_segsizepix = int(np.sqrt(2) * self.window_sizepix)
        
        micrograph_img, matched_mic_find, micrograph_name, log_info, ctf_info, each_image_index = \
        self.pre_loop_setup()
        
        for each_helix_id, each_helix in enumerate(helices):
            micrograph_img, matched_mic_find, micrograph_name = self.read_micrograph_and_get_ctf_info(micrograph_img,
            matched_mic_find, micrograph_name, session, each_helix)
            
            for each_segment_index, coordpair in enumerate(each_helix.coordinates):
                xcoord, ycoord = coordpair
                if (self.stepsize == 0 and each_segment_index == 0) or \
                (self.stepsize == 0 and each_segment_index == len(each_helix.coordinates) - 1):
                    pass
                else:
                    windowed_segment, ctf_info = self.window_segment_and_invert_normalize_and_ctf_correct(session,
                    large_segsizepix, each_helix, micrograph_img, matched_mic_find, xcoord, ycoord,
                    ctf_info, each_image_index)
                    
                    polyfit = self.rotate_and_unbend_segment_if_required(imgstack, each_image_index,
                    large_segsizepix, each_helix, each_segment_index, coordpair, windowed_segment)
                    
                    log_info += [[each_image_index, each_helix.helix_id, os.path.basename(micrograph_name), xcoord,
                    ycoord, self.normoption, self.invertoption, self.rotoption, self.unbending, polyfit.__str__()]]
                    
                    each_image_index = each_image_index + 1
            self.log.plog(10.0 + (each_helix_id + 1) * 70.0 / float(len(helices)))
        msg = tabulate(log_info, self.get_log_info_header())
        if self.ctfcorrect_option:
            msg += '\n' + tabulate(ctf_info, self.get_ctfinfo_header())
        self.log.ilog('The following segments have been windowed:\n{0}'.format(msg))
                    
        return helices
    
    
    def compare_centered_coordinates_with_picked_coordinates_and_smooth_path(self, helices):
        for each_helix_id, each_helix in enumerate(helices):
            x_coord, y_coord = zip(*each_helix.coordinates)
            x_coord = np.array(x_coord)
            y_coord = np.array(y_coord)
            
            picked_x_coord, picked_y_coord = zip(*each_helix.picked_coordinates)
            
            shifted_x = x_coord - np.array(picked_x_coord)
            shifted_y = y_coord - np.array(picked_y_coord)
            
            shifts_x_helix, shifts_y_helix = self.rotate_coordinates_by_angle(shifted_x, shifted_y,
            np.average(each_helix.inplane_angle))
            
            excessive_shift_indices = np.where(np.abs(shifts_x_helix) > 0.35*self.helixwidthpix)
            for each_shift_index in excessive_shift_indices[0]:
                x_coord[each_shift_index] = picked_x_coord[each_shift_index]
                y_coord[each_shift_index] = picked_y_coord[each_shift_index]
            
            interpolated_xcoord, interpolated_ycoord, ipangle, curvature = \
            self.interpolate_coordinates(np.array(x_coord), np.array(y_coord), self.pixelsize, self.stepsize,
            self.helixwidth, each_helix.segment_list, new_stepsize=False)
            
            helices[each_helix_id] = helices[each_helix_id]._replace(coordinates=zip(interpolated_xcoord,
            interpolated_ycoord))
            
            helices[each_helix_id] = helices[each_helix_id]._replace(inplane_angle=ipangle)
            helices[each_helix_id] = helices[each_helix_id]._replace(curvature=curvature)
        
        return helices
            

class SegmentDatabase(SegmentCut):
            
    def remove_all_previous_entries(self, session, column_name):
#        if session.query(column_name).count > 0:
        column = session.query(column_name).all()
        for each_item in column:
            session.delete(each_item)
        
        return session
    

    def enter_parameters_into_helix_table(self, session, each_helix, current_mic):
        matched_helix_id = session.query(HelixTable.id).\
        filter(HelixTable.helix_name == os.path.basename(each_helix.segment_list)).first()
        
        if matched_helix_id is not None:
            helix = session.query(HelixTable).get(matched_helix_id)
        else:
            helix = HelixTable()
            helix.helix_name = os.path.basename(each_helix.segment_list)
        helix.micrographs = current_mic
        helix.dirname = os.path.dirname(each_helix.segment_list)
        helix.avg_inplane_angle = sum(each_helix.inplane_angle) / float(len(each_helix.inplane_angle))
        helix.avg_curvature = sum(each_helix.curvature) / float(len(each_helix.curvature))
        if self.stepsize != 0 and self.averaging_option:
            window_size = int(self.averaging_distance / self.stepsize)
            lavg_inplane_angles = SegmentSelect().compute_local_average_from_measurements(each_helix.inplane_angle, window_size)
            lavg_curvatures = SegmentSelect().compute_local_average_from_measurements(each_helix.curvature, window_size)
            helix.lavg_distance_A = self.averaging_distance
        else:
            lavg_inplane_angles = each_helix.inplane_angle
            lavg_curvatures = each_helix.curvature
            helix.lavg_distance_A = 0
        previous_segment_ids_from_helix = session.query(SegmentTable).filter(SegmentTable.helix_id == helix.id).all()
        for each_previous_segment in previous_segment_ids_from_helix:
            session.delete(each_previous_segment)
        
        x_coordinates, y_coordinates = list(zip(*each_helix.coordinates))
        distances = self.compute_cumulative_distances_from_start_of_helix(x_coordinates, y_coordinates) * self.pixelsize
        helix.length = distances[-1]
        session.add(helix)
        
        return session, helix, distances, lavg_inplane_angles, lavg_curvatures
    

    def round_and_calculate_coordinate_in_Angstrom(self, each_helix_coordinate, pixelsize):
        coordinate_A = round(each_helix_coordinate) * pixelsize
        
        return coordinate_A
    

    def enter_parameters_into_segment_table(self, session, stack_id, each_helix, current_mic, helix, distances,
    lavg_inplane_angles, lavg_curvatures):
        if self.ctfcorrect_option:
            matched_mic_find = SegmentCtfApply().get_micrograph_from_database_by_micname(session, each_helix.micrograph,
            self.spring_path)
        for each_segment_index, each_helix_coordinate in enumerate(each_helix.coordinates):
            if self.stepsize == 0 and each_segment_index == 0 or self.stepsize != 0:
                segment = SegmentTable()
                segment.helices = helix
                segment.micrographs = current_mic
                segment.stack_id = stack_id
                x_coordinate = self.round_and_calculate_coordinate_in_Angstrom(each_helix_coordinate[0], self.pixelsize)
                y_coordinate = self.round_and_calculate_coordinate_in_Angstrom(each_helix_coordinate[1], self.pixelsize)
                segment.x_coordinate_A = x_coordinate
                segment.y_coordinate_A = y_coordinate
                
                picked_x_coordinate = \
                self.round_and_calculate_coordinate_in_Angstrom(each_helix.picked_coordinates[each_segment_index][0],
                self.pixelsize)
                
                picked_y_coordinate = \
                self.round_and_calculate_coordinate_in_Angstrom(each_helix.picked_coordinates[each_segment_index][1],
                self.pixelsize)
                
                segment.picked_x_coordinate_A = picked_x_coordinate
                segment.picked_y_coordinate_A = picked_y_coordinate
                segment.distance_from_start_A = distances[each_segment_index]
                segment.inplane_angle = each_helix.inplane_angle[each_segment_index]
                segment.curvature = each_helix.curvature[each_segment_index]
                segment.lavg_inplane_angle = lavg_inplane_angles[each_segment_index]
                segment.lavg_curvature = lavg_curvatures[each_segment_index]

                segment_info = self.make_segment_info_named_tuple()
                each_segment = segment_info._make([current_mic.id, x_coordinate, y_coordinate])
                if self.ctfcorrect_option:
                    ctf_params, avg_defocus, astigmatism, astig_angle = \
                    SegmentCtfApply().get_ctfparameters_from_database(self.ctffind_or_ctftilt_choice,
                    self.astigmatism_option, self.pixelsize, session, each_segment, matched_mic_find, self.spring_path)
                    
                    session, segment = \
                    SegmentCtfApply().update_ctfparameters_in_database(self.ctffind_or_ctftilt_choice,
                    self.convolve_or_phaseflip_choice, self.astigmatism_option, session, segment, avg_defocus,
                    astigmatism, astig_angle)

                if len(each_helix.coordinates) >= 3 and self.stepsize != 0:
                    polyfit, filtered_x, filtered_y = self.fit_square_function_path_of_coordinates(self.segsizepix *\
                    1.5, each_helix.coordinates, each_helix_coordinate, each_helix_coordinate,
                    each_helix.inplane_angle[each_segment_index])
                    
                    segment.second_order_fit = polyfit[0] * self.pixelsize
                else:
                    segment.second_order_fit = 0
                session.add(segment)
                stack_id += 1
        
        return session, stack_id
    

    def enter_helix_info_into_segments_and_helix_tables(self, helices, session):
        stack_id = 0
        for each_helix in helices:
            matched_mic_id = session.query(CtfMicrographTable.id).\
            filter(CtfMicrographTable.micrograph_name == os.path.basename(each_helix.micrograph)).first()

            if matched_mic_id is not None:
                current_mic = session.query(CtfMicrographTable).get(matched_mic_id)
            else:
                current_mic = CtfMicrographTable()
                current_mic.dirname = os.path.dirname(each_helix.micrograph)
                current_mic.micrograph_name = os.path.basename(each_helix.micrograph)
                current_mic.pixelsize = self.pixelsize
                session.add(current_mic)
            session, helix, distances, lavg_inplane_angles, lavg_curvatures = \
            self.enter_parameters_into_helix_table(session, each_helix, current_mic)

            session, stack_id = self.enter_parameters_into_segment_table(session, stack_id, each_helix, current_mic, 
            helix, distances, lavg_inplane_angles, lavg_curvatures)
        
        session.commit()

        return session
    

    def copy_micrograph_info_on_ctffind_and_ctftilt(self, assigned_mics, source_session, new_session):

        mic_columns = SpringDataBase().get_columns_from_table(CtfMicrographTable)
        find_columns = SpringDataBase().get_columns_from_table(CtfFindMicrographTable)
        tilt_columns = SpringDataBase().get_columns_from_table(CtfTiltMicrographTable)

        for each_mic, each_inp_mics in assigned_mics:
            matched_mic = source_session.query(CtfMicrographTable).\
            filter(CtfMicrographTable.micrograph_name.startswith(os.path.basename(os.path.splitext(each_mic)[0]))).first()

            matched_mic_find = source_session.query(CtfFindMicrographTable).get(matched_mic.id)
            mic_data = SpringDataBase().get_data_from_entry(mic_columns, matched_mic)
            if matched_mic_find is not None:
                find_data = SpringDataBase().get_data_from_entry(find_columns, matched_mic_find)

            matched_mic_tilt = source_session.query(CtfTiltMicrographTable).get(matched_mic.id)
            if matched_mic_tilt is not None:
                tilt_data = SpringDataBase().get_data_from_entry(tilt_columns, matched_mic_tilt)
            for each_inp in each_inp_mics:
                new_mic = CtfMicrographTable()
                [setattr(new_mic, each_col, mic_data[each_col]) for each_col in mic_columns if each_col != 'id']
                new_mic.micrograph_name = os.path.basename(each_inp)

                if matched_mic_find is not None:
                    new_find_mic = CtfFindMicrographTable()
                    [setattr(new_find_mic, each_col, find_data[each_col]) for each_col in find_columns if each_col != 'id']
                    new_find_mic.micrograph_name = os.path.basename(each_inp)
                    new_find_mic.ctf_micrographs = new_mic
                    
                    new_session.add(new_find_mic)

                if matched_mic_tilt is not None:
                    new_tilt_mic = CtfTiltMicrographTable()
                    [setattr(new_tilt_mic, each_col, tilt_data[each_col]) for each_col in tilt_columns if each_col != 'id']
                    new_tilt_mic.ctf_micrographs = new_mic
                    
                    new_session.add(new_tilt_mic)
                else:
                    new_session.add(new_mic)
        
        new_session.commit()
        
        return new_session


    def copy_and_duplicate_refined_segment_entries_to_new_database(self, assigned_stack_ids, ref_source_session,
    new_ref_session):

        ref_columns = SpringDataBase().get_columns_from_table(RefinementCycleSegmentTable)
        for each_related_stack_id, each_stack_id in enumerate(assigned_stack_ids):
            each_ref_entry = ref_source_session.query(RefinementCycleSegmentTable).\
            filter(RefinementCycleSegmentTable.stack_id == int(each_stack_id)).first()
            ref_data = SpringDataBase().get_data_from_entry(ref_columns, each_ref_entry)
            if ref_data is not None:
                ref_data['id'] = None
                ref_data['stack_id'] = each_related_stack_id
                new_ref_session.add(RefinementCycleSegmentTable(**ref_data))
        
        new_ref_session.commit()
        
        return new_ref_session


    def copy_and_duplicate_refined_helix_entries_to_new_database(self, assigned_helix_ids, ref_source_session,
    new_ref_session):
        hel_columns = SpringDataBase().get_columns_from_table(RefinementCycleHelixTable)
        for each_updated_hel_id, each_helix_id in enumerate(assigned_helix_ids):
            each_ref_entry = ref_source_session.query(RefinementCycleHelixTable).\
            filter(RefinementCycleHelixTable.helix_id == int(each_helix_id + 1)).first()

            hel_data = SpringDataBase().get_data_from_entry(hel_columns, each_ref_entry)
            if hel_data is not None:
                hel_data['id'] = None
                hel_data['helix_id']= each_updated_hel_id + 1
                new_ref_session.add(RefinementCycleHelixTable(**hel_data))
        
        new_ref_session.commit()
        
        return new_ref_session


    def copy_refinement_info_into_new_refinement_database(self, assigned_stack_ids, assigned_helix_ids):
        ref_source_session = SpringDataBase().setup_sqlite_db(refine_base, self.ref_db)
        new_ref_session = SpringDataBase().setup_sqlite_db(refine_base, self.ref_db_frames)

        new_ref_session = self.copy_and_duplicate_refined_segment_entries_to_new_database(assigned_stack_ids,
        ref_source_session, new_ref_session)

        new_ref_session = self.copy_and_duplicate_refined_helix_entries_to_new_database(assigned_helix_ids,
        ref_source_session, new_ref_session)

        new_ref_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(RefinementCycleTable, 
        new_ref_session, ref_source_session)


    def enter_helix_parameters_in_database(self, helices, assigned_stack_ids, assigned_helix_ids):
        if not self.frame_option:
            if self.spring_db_option:
                shutil.copy(self.spring_path, 'spring.db')
            session = SpringDataBase().setup_sqlite_db(base)
            
            if self.spring_db_option:
                session = SpringDataBase().remove_all_previous_entries(session, SegmentTable)
                session = SpringDataBase().remove_all_previous_entries(session, HelixTable)
                    
            session = self.enter_helix_info_into_segments_and_helix_tables(helices, session)
        
        if self.frame_option:
            new_session = SpringDataBase().setup_sqlite_db(base, self.spring_db_frames)

            new_session = self.enter_helix_info_into_segments_and_helix_tables(helices, new_session)
        
            self.copy_refinement_info_into_new_refinement_database(assigned_stack_ids, assigned_helix_ids)
        

class Segment(SegmentDatabase):
    def binstack(self, imgstack=None, binned_imgstack=None, binfactor=None):
        """
        * Function to bin image stack by factor

        #. Input: image stack to be binned, bining factor (1 = no binning)
        #. Output: binned stack
        #. Usage: binnedimgstack = binstack(imgstack, binfactor)
        
        """
        self.log.fcttolog()

        if imgstack is None: 
            imgstack = self.imgstack
        if binfactor is None: 
            binfactor = self.binfactor

        if binfactor < 2:
            self.log.errlog('Bin factor of {0} too low, binning is not required'.format(binfactor))
        else:
            segment = EMData()
            noimgstack = EMUtil.get_image_count(imgstack)
            for each_segment_index in range(noimgstack):
                segment.read_image(imgstack, each_segment_index)
                decsegment = image_decimate(segment, binfactor)
                decsegment.write_image(binned_imgstack, each_segment_index)
            self.log.ilog('{0} images from {1} were binned by a factor of {2}'.format(noimgstack, imgstack, binfactor))

        return binned_imgstack

    def segment(self):
        self.helices, assigned_stack_ids, assigned_helix_ids = self.prepare_segmentation()
        self.log.plog(10)
        
        imgstack = self.outfile
        self.helices = self.cut_segments(self.helices, imgstack)

        self.log.plog(80)

        self.enter_helix_parameters_in_database(self.helices, assigned_stack_ids, assigned_helix_ids)
        if self.binoption is True: 
            binned_imgstack = '{0}-{1}xbin{2}'.format(os.path.splitext(imgstack)[0], self.binfactor,
            os.path.splitext(imgstack)[-1])
            
            self.binstack(imgstack, binned_imgstack, self.binfactor)

        self.log.endlog(self.feature_set)
        
        return self.outfile

def main():
    # Option handling
    parset = SegmentPar()
    mergeparset = OptHandler(parset)
 
    ######## Program
    mic = Segment(mergeparset)
    mic.segment()

if __name__ == '__main__':
    main()



# Author: Carsten Sachse 6-Aug-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.particle2d.particlealign2d import ParticleAlign2dPar, \
    ParticleAlign2d
from spring.segment2d.segmentalign2d_mpi import SegmentAlign2dMpi

class ParticleAlign2dMpi(ParticleAlign2d, SegmentAlign2dMpi):
    pass

def main():
    parset = ParticleAlign2dPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    stack = ParticleAlign2dMpi(reduced_parset)
    stack.perform_segmentalign2d_mpi()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Dummy module to check seggridexplore
"""
from glob import glob
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment3d.seggridexplore import SegGridExplore, SegGridExplorePar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment3d.test_segclassreconstruct import TestSegClassReconstructMain
import os


class TestSegGridExplorePreparation(object):
    def setup_segclassreconstruct(self):
        self.tsrce = TestSegClassReconstructMain()
        self.tsrce.setup()
        self.tsrce.do_test_case_cr1()

    def setup_of_sgx_parameters(self):
        self.feature_set = SegGridExplorePar()
        
        self.pixelsize = 10.0
        self.reprojection_file = 'test_prefix.hdf'
        self.symmetry_grid_testfile = 'grid.db'
        
        self.feature_set.parameters['Grid database']=os.path.join(os.pardir, self.symmetry_grid_testfile)
        self.feature_set.parameters['Subgrid merge option']=False
        self.feature_set.parameters['Reconstitute Grid']=False
        self.feature_set.parameters['Batch mode']=True#False#
        self.feature_set.parameters['EM name']=self.reprojection_file
        self.feature_set.parameters['Parameter pair']=((10.0, 50.0))
            
        
class TestSegGridExplore(SegGridExplore, TestSegGridExplorePreparation):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.setup_segclassreconstruct()
        self.setup_of_sgx_parameters()
        os.rename(self.symmetry_grid_testfile, os.path.join(os.pardir, self.symmetry_grid_testfile))
        
        super(TestSegGridExplore, self).__init__(self.feature_set)
        
    def teardown(self):
        if self.batch_mode:
            files = glob('{0}*{1}'.format(os.path.splitext(self.reprojection_file)[0],
            os.path.splitext(self.reprojection_file)[-1]))
             
            for each_file in files:
                os.remove(each_file)
        self.tsrce.teardown()
        
        self.testingdir.remove()


class TestSegGridExploreMain(TestSegGridExplore):
    def do_test_sgx1(self):
        self.launch_seggridexplore()
        
class TestSegGridExploreEndToEnd(TestSegGridExplore):
    def do_end_to_end_test_sge_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 
    def do_end_to_end_test_sge_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
def main():
    ssx = TestSegGridExploreMain()
    ssx.setup()
    ssx.do_test_sgx1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import EMData, EMNumPy, Util, periodogram
from collections import namedtuple
from filter import filt_table
from fundamentals import ccfnpl, mirror
import os
from spring.csinfrastr.csdatabase import SpringDataBase, refine_base, RefinementCycleTable
from spring.csinfrastr.csgui import NumbersOptionsGuiWindow
from spring.csinfrastr.csproductivity import ExtLauncher
from spring.segment2d.segmentalign2d import SegmentAlign2d
from spring.segment2d.segmentexam import SegmentExam
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.springgui.springdataexplore import SpringCommon, SpringDataExplore

from PyQt4.QtCore import Qt, SIGNAL, SLOT, QThread
from PyQt4.QtGui import QWidget, QSplitter, QSlider, QGridLayout, QLabel, QPushButton, QDoubleSpinBox, QFileDialog, \
    QComboBox, QStackedWidget, QCheckBox
from sqlalchemy.sql.expression import desc

import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
from textwrap import wrap


class SegRefine3dInspectCommonOperations(object):
    def convert_helical_symmetry_from_pitch_unit_number_to_rise_rotation(self, rise_or_pitch_choice, helical_symmetry):
        if rise_or_pitch_choice == 'pitch/unit_number':
            pitch, unit_number = helical_symmetry
            helical_symmetry = SegClassReconstruct().convert_pitch_unit_pair_to_rise_rotation_pairs(pitch, unit_number)
        
        return helical_symmetry
    

    def make_volume_collection_named_tuple(self):
        volume_collection = namedtuple('volumes', 'original layer cylinder_mask structural_mask')
        
        return volume_collection


    def apply_bfactor_and_resolution_cutoff(self, vol, bfactor, res_cutoff, pixelsize, fsc_line):
        volume_size = vol.get_xsize()

        filter_coefficients = SegmentAlign2d().prepare_bfactor_coefficients(bfactor, pixelsize, volume_size, res_cutoff)
        if fsc_line is not None:
            filter_coefficients *= np.sqrt(2 * fsc_line / (1 + fsc_line))
        
        vol = filt_table(vol, filter_coefficients.tolist())
        
        return vol
    

    def read_columns_from_text_file(self, fsc_file):
        f = open(fsc_file, 'r')
        fsc_line = []
        for each_row, each_line in enumerate(f.readlines()):
            if each_row == 0 and not each_line.strip().startswith('resolution') or \
            each_row == 1 and not each_line.strip().startswith('----------'):
                msg = 'This is not an FSC file generated from segmentrefine3d. Header is not recognized.'
                raise ValueError(msg)
            elif each_row > 1:
                fsc_line.append(float(each_line.split()[2]))
        
        return fsc_line


    def read_fsc_from_refinement_db(self, fsc_file):
        ref_session = SpringDataBase().setup_sqlite_db(refine_base, fsc_file)
        
        last_cycle = ref_session.query(RefinementCycleTable).order_by(desc(RefinementCycleTable.id)).first()
        
        return last_cycle.fsc


    def read_fsc_line_from_file(self, fsc_file, vol_size, pad_dim=False):
        if fsc_file.endswith('dat'):
            fsc_line = self.read_columns_from_text_file(fsc_file)
        elif fsc_file.endswith('db'):
            fsc_line = self.read_fsc_from_refinement_db(fsc_file)
        
        fsc_line = np.abs(fsc_line)
        if len(fsc_line) == vol_size / 2 + 1:
            fsc_line = np.array(fsc_line[1:])
        
        if not pad_dim and len(fsc_line) != vol_size / 2:
            msg = 'Dimension of volume and number of fsc line values do not agree. Please double check whether ' + \
            'files are from the same cycle of segmentrefine3d.'
            raise ValueError(msg)
        
        return fsc_line

            
    def prepare_long_helix(self, vol, helix_length, helixwidth, pixelsize, helical_symmetry, rotational_symmetry,
    polar_helix):
        xy_size = int(round(1.2 * helixwidth / pixelsize, -1))
        z_size = int(round(helix_length / pixelsize, -1))

        point_symmetry = SegClassReconstruct().determine_point_group_symmetry_from_input(polar_helix, 
        rotational_symmetry)

        vol = SegmentRefine3d().generate_long_helix_volume(vol, xy_size, z_size, helical_symmetry, 
        pixelsize, point_symmetry)
        
        return vol
        

    def fit_bfactor_from_sphere_avg_amp(self, sphere_avg_amp, recip_res, high_res_cutoff, low_res_cutoff=10.0):
        """
        >>> from spring.segment3d.segrefine3dinspect_gui import SegRefine3dInspectCommonOperations
        >>> from spring.segment2d.segmentexam import SegmentExam
        >>> s = SegRefine3dInspectCommonOperations()
        >>> pixelsize = 2.0
        >>> sphere_avg_amp = SegmentAlign2d().prepare_bfactor_coefficients(500, pixelsize, 200)
        >>> recip_res = SegmentExam().make_oneoverres(sphere_avg_amp, pixelsize)
        >>> s.fit_bfactor_from_sphere_avg_amp(sphere_avg_amp, recip_res, 5.0)
        (500.00000000000011, 6.7053176943786003e-29)
        """

        start_id = np.argmin(np.abs(recip_res - 1 / low_res_cutoff))
        stop_id = np.argmin(np.abs(recip_res - 1 / high_res_cutoff))

        sphere_avg_amp_log = np.log(sphere_avg_amp[start_id:stop_id])
        recip_res_sq = recip_res[start_id:stop_id] ** 2
        
        start_id = np.argmin(np.abs(recip_res - 1 / low_res_cutoff))
        stop_id = np.argmin(np.abs(recip_res - 1 / high_res_cutoff))

        coefs = np.polyfit(recip_res_sq, sphere_avg_amp_log, 1)
        bfactor = -4 * coefs[0]
        yfit = np.polyval(coefs, recip_res_sq)
        residual = np.sum((sphere_avg_amp_log - yfit)**2)
        
        return bfactor, residual


    def estimate_bfactor_from_vol(self, vol, pixelsize, high_res_cutoff, low_res_cutoff=10.0):
        pvol = periodogram(vol)
        sphere_avg_amp = np.copy(EMNumPy.em2numpy(pvol.rotavg()))
        
        recip_res = SegmentExam().make_oneoverres(sphere_avg_amp, pixelsize)
        bfactor, resid = self.fit_bfactor_from_sphere_avg_amp(sphere_avg_amp, recip_res, high_res_cutoff, low_res_cutoff)

        return bfactor, resid
        

class SegRefine3dInspectGuiSliceView(QWidget, QThread):
    def __init__(self, volume, plane, label, figure_no, parent = None):
        QWidget.__init__(self, parent) 
        QThread.__init__(self, parent) 
        
        self.volume = volume
        self.slice_plane = plane
        self.label = label
        self.figure_no = figure_no
    
        self.layout = QGridLayout()
        
        self.fig = self.build_initial_slice_fig()
        self.canvas = SpringDataExplore(self.fig)
        self.layout.addWidget(self.canvas, 0, 0, 1, 3)
        
        self.setLayout(self.layout)
        
    
    def build_initial_slice_fig(self):
        self.fig = plt.figure(self.figure_no)
        self.ax = self.fig.add_subplot(111)
        self.slice_no_z = self.volume.shape[0]/2
        self.slice = self.ax.imshow(self.volume[:,self.slice_no_z], cmap=cm.jet, origin='upper')
        
        cax = self.fig.add_axes()
        cbar = self.fig.colorbar(self.slice, cax)
        cbar.set_label('Density (normalized)')

        if self.label in ['Radius (Angstrom)', 'Theta (degrees)', 'Z distance (Angstrom)']:
            self.slice.set_clim(self.volume[self.slice_no_z].min(), self.volume[self.slice_no_z].max())
        else:
            self.slice.set_clim(self.volume[:,self.slice_no_z].min(), self.volume[:,self.slice_no_z].max())
#        self.slice = self.ax.imshow(self.volume[self.slice_no_z], interpolation='nearest', cmap=cm.gray, origin='lower')

        return self.fig
    
        
class SegRefine3dInspectGuiSlice(QWidget, QThread):
    def __init__(self, volume, label, figure_no, parent = None):
        QWidget.__init__(self, parent)
        QThread.__init__(self, parent)
    
        self.volume = volume
        self.label = label
        self.figure_no = figure_no
        
        self.build_three_slice_planes()
        self.setLayout(self.layout)
        
    def build_autocorr_volume(self):
        self.auto_volume = np.zeros(self.volume.shape)
        x_count, y_count = self.volume[:,:,0].shape
        for each_x_plane in list(range(x_count)):
            z_slice = EMNumPy.numpy2em(np.copy(self.volume[:,each_x_plane]))
#            ccfnpl(z_slice, z_slice).write_image('autocorr.hdf', each_x_plane)
            slice_auto = np.copy(EMNumPy.em2numpy(ccfnpl(z_slice, z_slice)))
            
            self.auto_volume[:,each_x_plane]=slice_auto
            
        return self.auto_volume
    
            
    def build_three_slice_planes(self):
        self.layout = QGridLayout()
        self.label_field = QLabel()
        self.label_field.setText(self.label.title)
        self.layout.addWidget(self.label_field, 0, 0, 1, 3)
        
        self.slider = QSlider(Qt.Horizontal, self)
        self.slider.setFocusPolicy(Qt.NoFocus)
        self.slider.setGeometry(30, 40, 100, 30)
        self.slider.setValue(50)
        self.slider.valueChanged[int].connect(self.changeSliceNumber)
        self.layout.addWidget(self.slider, 1, 0, 1, 2)
        
        self.minmax_label = QLabel()
        self.minmax_label.setText(' - '.join([str(int(each_label)) for each_label in self.label.z_min_max]))
        self.minmax_label.setAlignment(Qt.AlignRight | Qt.AlignCenter)
        
        self.layout.addWidget(self.minmax_label, 1, 2, 1, 1)
        self.spinbox = QDoubleSpinBox()
        self.spinbox.valueChanged[float].connect(self.changeSliceNumberSpinBox)
        self.layout.addWidget(self.spinbox, 1, 3, 1, 1)
        
        self.stacked_slice_viewer = QStackedWidget()
        self.x_plane_viewer = SegRefine3dInspectGuiSliceView(self.volume, 0, self.label.label_x, 1 + 10 * self.figure_no)
        self.y_plane_viewer = SegRefine3dInspectGuiSliceView(self.volume, 1, self.label.label_y, 2 + 10 * self.figure_no)
        self.z_plane_viewer = SegRefine3dInspectGuiSliceView(self.volume, 2, self.label.label_z, 0 + 10 * self.figure_no)
        self.auto_volume = self.build_autocorr_volume()
        
        self.x_auto_viewer = SegRefine3dInspectGuiSliceView(self.auto_volume, 0, 'auto' + self.label.label_x, 3 + \
        10 * self.figure_no)
            
        self.stacked_slice_viewer.addWidget(self.x_plane_viewer)
        self.stacked_slice_viewer.addWidget(self.y_plane_viewer)
        self.stacked_slice_viewer.addWidget(self.z_plane_viewer)
        self.stacked_slice_viewer.addWidget(self.x_auto_viewer)
        self.layout.addWidget(self.stacked_slice_viewer, 2, 0, 4, 4)
        
        self.slice_direction = 2
        self.slice_choice = QComboBox()
        
        self.slice_choice.addItems([self.label.label_x, self.label.label_y, self.label.label_z, 'Autocorrelation {' + \
        self.label.label_x + '}'])
        
        self.connect(self.slice_choice, SIGNAL('currentIndexChanged(int)'), self.update_slice_choice)
        self.layout.addWidget(self.slice_choice, 0, 3, 1, 1)
        
        for each_slice in list(range(self.slice_direction)):
            self.update_slice_choice(each_slice)
        self.slice_choice.setCurrentIndex(self.slice_direction)
        
    def update_slice_choice(self, value):
        self.slice_plane = value
        self.set_ticks_and_labels_for_slices()
        self.stacked_slice_viewer.setCurrentIndex(value)
        
    def set_ticks_and_labels_for_slices(self):
        if self.slice_plane == 0:
            x_count, y_count = self.volume[:,:,0].shape
            z_count = self.volume.shape[2]
            x_y_label = [self.label.label_z, self.label.label_y]
            x_min_max, y_min_max  = self.label.z_min_max, self.label.y_min_max
            self.z_min_max = self.label.x_min_max
            self.label_z_text = self.label.label_x
            
            self.x_plane_viewer.canvas.set_adjustable_tick_values(self.x_plane_viewer.ax, x_count, y_count, x_min_max,
            y_min_max)
            
            self.x_plane_viewer.canvas.set_x_and_y_label(self.x_plane_viewer.ax, x_y_label)
            self.x_plane_viewer.slice.axes.figure.canvas.draw()
        elif self.slice_plane == 1:
            x_count, y_count = self.volume[:,0].shape
            z_count = self.volume.shape[1]
            x_y_label = [self.label.label_z, self.label.label_x]
            x_min_max, y_min_max  = self.label.z_min_max, self.label.x_min_max
            self.z_min_max = self.label.y_min_max
            self.label_z_text = self.label.label_y
            
            self.y_plane_viewer.canvas.set_adjustable_tick_values(self.y_plane_viewer.ax, x_count, y_count, x_min_max,
            y_min_max)
            
            self.y_plane_viewer.canvas.set_x_and_y_label(self.y_plane_viewer.ax, x_y_label)
            self.y_plane_viewer.slice.axes.figure.canvas.draw()
        elif self.slice_plane == 2:
            x_count, y_count = self.volume[0].shape
            z_count = self.volume.shape[0]
            x_y_label = [self.label.label_x, self.label.label_y]
            x_min_max, y_min_max  = self.label.x_min_max, self.label.y_min_max
            self.z_min_max = self.label.z_min_max
            self.label_z_text = self.label.label_z
            
            self.z_plane_viewer.canvas.set_adjustable_tick_values(self.z_plane_viewer.ax, x_count, y_count, x_min_max,
            y_min_max)
            
            self.z_plane_viewer.canvas.set_x_and_y_label(self.z_plane_viewer.ax, x_y_label)
            self.z_plane_viewer.slice.axes.figure.canvas.draw()
        if self.slice_plane == 3:
            x_count, y_count = self.volume[:,:,0].shape
            z_count = self.volume.shape[2]
            x_y_label = [self.label.label_z, self.label.label_y]
            x_min_max, y_min_max  = self.label.z_min_max, self.label.y_min_max
            self.z_min_max = self.label.x_min_max
            self.label_z_text = 'Autocorrelation {' + self.label.label_x + '}'
            
            self.x_auto_viewer.canvas.set_adjustable_tick_values(self.x_auto_viewer.ax, x_count, y_count, x_min_max,
            y_min_max)
            
            self.x_auto_viewer.canvas.set_x_and_y_label(self.x_auto_viewer.ax, x_y_label)
            self.x_auto_viewer.slice.axes.figure.canvas.draw()
            
        self.minmax_label.setText(' - '.join([str(int(each_label)) for each_label in self.z_min_max]))
        value = self.slider.value()
        self.spinbox.setValue(self.z_min_max[0] + value/99.0 * (self.z_min_max[1] - self.z_min_max[0]))
        self.changeSliceNumber(value)
        
        self.spinbox.setRange(self.z_min_max[0], self.z_min_max[1])
        step = (self.z_min_max[1] - self.z_min_max[0]) / float(z_count)
        self.spinbox.setSingleStep(step)
        self.spinbox.setDecimals(1)
        
        
    def changeSliceNumberSpinBox(self, value):
        slider_value = int(round(99 * (value - self.z_min_max[0]) / float(self.z_min_max[1] - self.z_min_max[0])))
        self.slider.setValue(slider_value)        
#        sys.stderr.write('Changed {0}'.format(slider_value))
    
    def changeSliceNumber(self, value):
#        sys.stderr.write('Changed {0}'.format(value))
        if self.slice_plane == 0:
            self.slider_value_x = value
            self.slice_no_x = int(value/100.0 * self.volume.shape[1])
            self.x_plane_viewer.slice.set_data(self.volume[:,self.slice_no_x])
            self.x_plane_viewer.slice.axes.figure.canvas.draw()
        elif self.slice_plane == 1:
            self.slider_value_y = value
            self.slice_no_y = int(value/100.0 * self.volume.shape[2])
            self.y_plane_viewer.slice.set_data(self.volume[:,:,self.slice_no_y])
            self.y_plane_viewer.slice.axes.figure.canvas.draw()
        elif self.slice_plane == 2:
            self.slider_value_z = value
            self.slice_no_z = int(value/100.0 * self.volume.shape[0])
            self.z_plane_viewer.slice.set_data(self.volume[self.slice_no_z])
            self.z_plane_viewer.slice.axes.figure.canvas.draw()
        if self.slice_plane == 3:
            self.slider_value_x = value
            self.slice_no_x = int(value/100.0 * self.volume.shape[1])
            self.x_auto_viewer.slice.set_data(self.auto_volume[:,self.slice_no_x])
            self.x_auto_viewer.slice.axes.figure.canvas.draw()
        
        self.spinbox.setValue(self.z_min_max[0] + value/99.0 * (self.z_min_max[1] - self.z_min_max[0]))
        
    def update_volume_and_views(self, volume):
        self.volume = volume
        
        self.x_plane_viewer.slice.set_data(self.volume[:,self.slice_no_x])
        self.x_plane_viewer.slice.set_clim(0, self.volume.max())
        self.x_plane_viewer.slice.axes.figure.canvas.draw()
        
        self.y_plane_viewer.slice.set_data(self.volume[:,:,self.slice_no_y])
        self.y_plane_viewer.slice.set_clim(0, self.volume.max())
        self.y_plane_viewer.slice.axes.figure.canvas.draw()
        
        self.z_plane_viewer.slice.set_data(self.volume[self.slice_no_z])
        self.z_plane_viewer.slice.set_clim(0, self.volume.max())
        self.z_plane_viewer.slice.axes.figure.canvas.draw()
        
        self.auto_volume = self.build_autocorr_volume()

        self.x_auto_viewer.slice.set_data(self.auto_volume[:,self.slice_no_x])
        self.x_auto_viewer.slice.axes.figure.canvas.draw()
        
            
class SegRefine3dInspectGuiItems(object):

    def build_original_volume_display(self):
        self.cartesian_window_ori = SegRefine3dInspectGuiSlice(self.current_vol, self.cart_label, 0)
        self.cylindrical_window_ori = SegRefine3dInspectGuiSlice(self.current_cyl_vol, self.cyl_label, 1)
        self.splitter_ori = QSplitter(Qt.Horizontal)
        self.splitter_ori.addWidget(self.cartesian_window_ori)
        self.splitter_ori.addWidget(self.cylindrical_window_ori)
        
        
    def build_mirror_check(self):
        self.mirror_check = QCheckBox()
        swap_handedness = 'Swap handedness'
        self.mirror_check.setText(swap_handedness)
        self.mirror_check.setToolTip(self.feature_set.hints[swap_handedness])
        self.layout.addWidget(self.mirror_check, 3, 0, 1, 1)
        self.connect(self.mirror_check, SIGNAL('toggled(bool)'), self.update_current_volume_with_mirror)
        
    
    def build_sn_weighting_check(self):
        if self.feature_set.parameters['Signal-to-noise weighting']:
            self.sn_weighting_check = QCheckBox()
            sn_weighting = 'Signal-to-noise weighting'
            self.sn_weighting_check.setText(sn_weighting)
            self.sn_weighting_check.setToolTip(self.feature_set.hints[sn_weighting])
            self.layout.addWidget(self.sn_weighting_check, 4, 3, 1, 1)
            self.connect(self.sn_weighting_check, SIGNAL('toggled(bool)'), self.update_current_fsc_line)
            
            self.fsc_line_read = \
            SegRefine3dInspectCommonOperations().read_fsc_line_from_file(self.feature_set.parameters['FSC curve'],
            self.current_vol_ref.get_xsize())
            self.fsc_line = self.fsc_line_read

        else:
            self.fsc_line_read = None
            self.fsc_line = None
    
    
    def build_bfactor_and_cutoff_dials(self):
        bfactor_res = 'B-factor and resolution cutoff'
        
        self.bfactor_check = QCheckBox()
        self.layout.addWidget(self.bfactor_check, 4, 0, 1, 1)
        self.bfactor_check.setText('{0}'.format(bfactor_res))
        self.last_bfactor_state = self.bfactor_check.checkState()

        self.bfactor_dial = QDoubleSpinBox()
        bfact_cutoff_tip = '{0} Confirm entry by pressing enter.'.format(self.feature_set.hints[bfactor_res])
        self.bfactor_dial.setToolTip(bfact_cutoff_tip)
        self = NumbersOptionsGuiWindow().compute_number_of_digits_from_properties(self, bfactor_res)
        self.bfactor_dial.setRange(self.properties[bfactor_res].minimum, self.properties[bfactor_res].maximum)
        self.bfactor_dial.setSingleStep(25)
        self.bfactor_dial.setDecimals(self.digits)
        self.bfactor_dial.setValue(self.feature_set.parameters[bfactor_res][0])
        self.connect(self.bfactor_dial, SIGNAL('editingFinished()'), self.activateOrInactivateBfactor)
        self.bfactor_dial.setEnabled(False)
        self.layout.addWidget(self.bfactor_dial, 4, 1, 1, 1)
        
        self.cutoff_dial = QDoubleSpinBox()
        self.cutoff_dial.setToolTip(bfact_cutoff_tip)
        self.cutoff_dial.setRange(0, self.properties[bfactor_res].maximum)
        self.cutoff_dial.setSingleStep(1)
        self.cutoff_dial.setDecimals(1)
        self.cutoff_dial.setValue(self.feature_set.parameters[bfactor_res][1])
        self.connect(self.cutoff_dial, SIGNAL('editingFinished()'), self.activateOrInactivateBfactor)
        self.cutoff_dial.setEnabled(False)
        self.layout.addWidget(self.cutoff_dial, 4, 2, 1, 1)

        self.connect(self.bfactor_check, SIGNAL('toggled(bool)'), self.cutoff_dial, SLOT('setEnabled(bool)'))
        self.connect(self.bfactor_check, SIGNAL('toggled(bool)'), self.bfactor_dial, SLOT('setEnabled(bool)'))
        self.connect(self.bfactor_check, SIGNAL('toggled(bool)'), self.activateOrInactivateBfactor)
        
        
    def build_structural_mask_choice(self):
        self.mask_choice = QComboBox()
        mask_type = 'Mask type'
        self.mask_label = QLabel()
        self.mask_label.setText(mask_type)
        items = ['None'] + \
        [each_choice.capitalize() for each_choice in self.feature_set.properties[mask_type].choices]
        self.mask_choice.addItems(items)
        self.mask_choice.setToolTip(self.feature_set.hints[mask_type])
        self.layout.addWidget(self.mask_label, 6, 0, 1, 1)
        self.layout.addWidget(self.mask_choice, 6, 1, 1, 1)
        self.connect(self.mask_choice, SIGNAL('currentIndexChanged(int)'), self.setCurrentDisplay)
        
        self.item_choice = dict([(each_item, each_index) for each_index, each_item in enumerate(items)])
    
        
    def build_layer_line_filter_check(self):
        self.layer_line_check = QCheckBox()
        layer_line_filter = 'Layer-line Fourier filter'
        self.layer_line_check.setText(layer_line_filter)
        self.layer_line_check.setToolTip(self.feature_set.hints[layer_line_filter])
        self.layout.addWidget(self.layer_line_check, 5, 0, 1, 1)
        if self.feature_set.parameters['Layer-line Fourier filter']:
            self.connect(self.layer_line_check, SIGNAL('toggled(bool)'), self.setCurrentDisplay)
        else:
            self.layer_line_check.setEnabled(False)
        
    
    def build_long_helix_buttons(self):
        self.helix_length_dial = QDoubleSpinBox()
        length = 'Helix length in Angstrom'
        self.helix_length_dial.setToolTip(self.feature_set.hints[length])
        self = NumbersOptionsGuiWindow().compute_number_of_digits_from_properties(self, length)
        self.helix_length_dial.setRange(self.properties[length].minimum, self.properties[length].maximum)
        self.helix_length_dial.setValue(self.feature_set.parameters[length])
        self.helix_length_dial.setSingleStep(self.properties[length].step)
        self.helix_length_dial.setDecimals(self.digits)
        self.helix_length_dial.setEnabled(False)
        self.layout.addWidget(self.helix_length_dial, 5, 4, 1, 1)
        
        self.long_helix_tick = QCheckBox()
        long_helix = 'Long helix'
        self.long_helix_tick.setText(long_helix)
        self.long_helix_tick.setToolTip('\n'.join(wrap(self.feature_set.hints[long_helix])))
        self.connect(self.long_helix_tick, SIGNAL('toggled(bool)'), self.helix_length_dial, SLOT('setEnabled(bool)'))
        self.connect(self.long_helix_tick, SIGNAL('toggled(bool)'), self.control_bfactor_check)
        self.connect(self.long_helix_tick, SIGNAL('toggled(bool)'), self.bfactor_check, SLOT('setDisabled(bool)'))
        self.connect(self.long_helix_tick, SIGNAL('toggled(bool)'), self.bfactor_dial, SLOT('setDisabled(bool)'))
        self.connect(self.long_helix_tick, SIGNAL('toggled(bool)'), self.cutoff_dial, SLOT('setDisabled(bool)'))
        self.layout.addWidget(self.long_helix_tick, 5, 3, 1, 1)
        

    def control_bfactor_check(self):
        if self.long_helix_tick.checkState() == 2:
            self.last_bfactor_state = self.bfactor_check.checkState()
            self.bfactor_check.setChecked(False)
        if self.long_helix_tick.checkState() == 0:
            self.bfactor_check.setChecked(self.last_bfactor_state)

        
    def build_open_button(self):
        self.open_button = QPushButton()
        self.open_button.setText('Open')
        self.open_button.setToolTip('Open volume in 3D viewer.')
        self.open_button.setShortcut('Ctrl+O')
        self.connect(self.open_button, SIGNAL('clicked()'), self.openVolume)
        self.layout.addWidget(self.open_button, 6, 3, 1, 1)
        

    def build_save_button(self):
        self.save_button = QPushButton()
        self.save_button.setText('Save')
        self.save_button.setToolTip('Save volume in current masked and filtered state.')
        self.save_button.setShortcut('Ctrl+S')
        self.connect(self.save_button, SIGNAL('clicked()'), self.saveVolume)
        self.layout.addWidget(self.save_button, 6, 4, 1, 1)
        

class SegRefine3dInspectGui(QWidget, SegRefine3dInspectGuiItems):

    def __init__(self, feature_set, em_volumes, parent = None):
        QWidget.__init__(self, parent)
        
        self.em_volumes = em_volumes
        self.cart_volumes = self.make_cart_volumes()
        self.current_vol_ref = self.em_volumes.original
        self.current_vol = np.copy(EMNumPy.em2numpy(self.em_volumes.original))
        
        self.current_cyl_vol, self.r, self.theta = SegmentRefine3d().reproject_volume_into_cylinder(self.current_vol)
        
        self.feature_set = feature_set
        self.properties = feature_set.properties
        self = SpringCommon().setup_spring_page_top(self, self.feature_set)
        
        self.get_parameters_from_dict()
        self.make_labels()
        self.build_original_volume_display()
        
        self.layout.addWidget(self.splitter_ori, 2, 0, 2, 5)
        
        self.build_mirror_check()
        self.build_bfactor_and_cutoff_dials()
        self.build_layer_line_filter_check()
        self.build_structural_mask_choice()
        self.build_sn_weighting_check()

        self.build_open_button()
        self.build_long_helix_buttons()
        self.build_save_button()
        self.setLayout(self.layout)
        
        self.external_launcher = ExtLauncher()

        self.setCurrentDisplay()
        
        self.set_values_from_parameters()

    def make_cart_volumes(self):
        self.volume_collection = SegRefine3dInspectCommonOperations().make_volume_collection_named_tuple()
        np_vols = [np.copy(EMNumPy.em2numpy(each_volume)) if each_volume is not None else None \
                   for each_volume in list(self.em_volumes)]
        self.cart_volumes = self.volume_collection._make(np_vols)
        
        return self.cart_volumes
        

    def get_parameters_from_dict(self):
        self.pixelsize = self.feature_set.parameters['Pixel size in Angstrom']
        self.helixwidth = self.feature_set.parameters['Estimated helix inner and outer diameter in Angstrom'][1]
        rise_or_pitch_choice = self.feature_set.parameters['Helical rise/rotation or pitch/number of units per turn choice']
        self.helical_symmetry = self.feature_set.parameters['Helical symmetry in Angstrom or degrees']
        self.rotational_symmetry = self.feature_set.parameters['Rotational symmetry']
        self.polar_helix = self.feature_set.parameters['Helix polarity']
        
        self.helical_symmetry = \
        SegRefine3dInspectCommonOperations().convert_helical_symmetry_from_pitch_unit_number_to_rise_rotation(
        rise_or_pitch_choice, self.helical_symmetry)


    def make_labels(self):
        label_collection = namedtuple('label', 'title label_x label_y label_z quantities x_min_max y_min_max z_min_max')
        distance = 'distance (Angstrom)'
        self.bfactor = 0.0
        self.res_cutoff_A = 0.0
        
        self.cart_label = label_collection('Cartesian image slices', 'X ' + distance, 'Y ' + distance, 'Z ' + distance, 
        ['x', 'y', 'z'], 
        [-self.pixelsize * self.cart_volumes[0].shape[1]/2, self.pixelsize * self.cart_volumes[0].shape[1]/2],
        [-self.pixelsize * self.cart_volumes[0].shape[2]/2, self.pixelsize * self.cart_volumes[0].shape[2]/2],
        [-self.pixelsize * self.cart_volumes[0].shape[0]/2, self.pixelsize * self.cart_volumes[0].shape[0]/2])
        
        self.cyl_label = label_collection('Cylindrical image slices', 'Radius (Angstrom)', 'Theta (degrees)', 
        'Z ' + distance, ['r', 'theta', 'z'],
        [self.pixelsize * self.r.min(), self.pixelsize * self.r.max()], 
#        [-180.0, 180.0], 
#        [np.rad2deg(self.theta.min()), np.rad2deg(self.theta.max())], 
        [np.rad2deg(self.theta.min()), 180.0], 
        [-self.pixelsize * self.cart_volumes[0].shape[0]/2, self.pixelsize * self.cart_volumes[0].shape[0]/2])
            

    def mirror_numpy_volume(self):
        current_vol = EMNumPy.numpy2em(np.copy(self.current_vol))
        current_vol = mirror(current_vol)
        self.current_vol = np.copy(EMNumPy.em2numpy(current_vol))
        

    def update_current_fsc_line(self):
        if self.sn_weighting_check.isChecked():
            self.fsc_line = self.fsc_line_read
        else:
            self.fsc_line = None

        self.setCurrentDisplay()


    def update_current_volume_with_mirror(self):
        self.mirror_numpy_volume()
        
        rise, rotation = self.helical_symmetry
        self.helical_symmetry = (rise, -rotation)
        
        self.current_cyl_vol, self.r, self.theta = \
        SegmentRefine3d().reproject_volume_into_cylinder(self.current_vol)
        
        self.update_views_of_volumes()
        
        
    def update_volume_to_numpy_and_cylinder_projection(self):
        self.current_vol = np.copy(EMNumPy.em2numpy(self.current_vol_ref))
        if self.mirror_check.isChecked():
            self.mirror_numpy_volume()
        
        self.current_cyl_vol, self.r, self.theta = \
        SegmentRefine3d().reproject_volume_into_cylinder(self.current_vol)
        
        self.update_views_of_volumes()
        

    def update_views_of_volumes(self):
        self.cartesian_window_ori.update_volume_and_views(self.current_vol)
        self.cylindrical_window_ori.update_volume_and_views(self.current_cyl_vol)
        
        
    def setCurrentDisplay(self):
        if self.layer_line_check.checkState() == 0:
            self.current_vol_ref = self.em_volumes.original
        elif self.layer_line_check.checkState() == 2:
            self.current_vol_ref = self.em_volumes.layer
        
        if self.bfactor != 0 and self.res_cutoff_A != 0:
            self.current_vol_ref = \
            SegRefine3dInspectCommonOperations().apply_bfactor_and_resolution_cutoff(self.current_vol_ref, self.bfactor,
            self.res_cutoff_A, self.pixelsize, self.fsc_line)
        
        if self.mask_choice.currentIndex() == 1:
            self.current_vol_ref *= self.em_volumes.cylinder_mask 
        elif self.mask_choice.currentIndex() == 2:
            self.current_vol_ref *= self.em_volumes.structural_mask 
            
        self.current_vol_ref.process_inplace('normalize')
        self.update_volume_to_numpy_and_cylinder_projection()
        

    def set_values_from_parameters(self):
        if self.feature_set.parameters['Swap handedness']:
            self.mirror_check.setChecked(True)
        if self.feature_set.parameters['B-factor']:
            self.bfactor_check.setChecked(True)
        if self.feature_set.parameters['Signal-to-noise weighting']:
            self.sn_weighting_check.setChecked(True)
        if self.feature_set.parameters['Layer-line Fourier filter']:
            self.layer_line_check.setChecked(True)
        if self.feature_set.parameters['Long helix']:
            self.long_helix_tick.setChecked(True)
        self.mask_choice.setCurrentIndex(self.item_choice[self.feature_set.parameters['Mask type'].capitalize()])


    def activateOrInactivateBfactor(self):
        if self.bfactor != self.bfactor_dial.value() or self.res_cutoff_A != self.cutoff_dial.value():
            if self.bfactor_check.checkState() == 2:
                self.bfactor = self.bfactor_dial.value()
                self.res_cutoff_A = self.cutoff_dial.value()
            self.setCurrentDisplay()
        elif self.bfactor_check.checkState() == 0:
            self.bfactor = 0.0
            self.res_cutoff_A = 0.0
            self.setCurrentDisplay()
        elif self.bfactor == self.bfactor_dial.value() and self.res_cutoff_A == self.cutoff_dial.value():
            pass 
        

    def if_input_volume_was_padded_trim_back_to_original_dimensions(self, vol):
        original_vol = EMData()
        original_vol.read_image(self.feature_set.parameters['Volume reconstruction'])

        xdim, ydim, zdim = original_vol.get_xsize(), original_vol.get_ysize(), original_vol.get_zsize()
        if xdim < vol.get_xsize() and ydim < vol.get_ysize() and zdim == vol.get_zsize():
            vol = Util.window(vol, xdim, ydim, zdim)

        return vol


    def get_cur_volume_and_write_out(self, fname):
        if fname != '':
            vol = EMNumPy.numpy2em(np.copy(self.current_vol))

            vol = SegClassReconstruct().set_isotropic_pixelsize_in_volume(self.pixelsize, vol)
            if self.long_helix_tick.isChecked():
                length = self.helix_length_dial.value()

                vol = SegRefine3dInspectCommonOperations().prepare_long_helix(vol, length, self.helixwidth,
                self.pixelsize, self.helical_symmetry, self.rotational_symmetry, self.polar_helix)

                vol.process_inplace('normalize')
            elif not self.long_helix_tick.isChecked():
                vol = self.if_input_volume_was_padded_trim_back_to_original_dimensions(vol)

            vol.write_image(fname)


    def saveVolume(self):
        fname = str(QFileDialog.getSaveFileName(self , 'Save volume', '.', 'Files ({0})'.\
        format('*.' + ' *.'.join(self.properties.values()[0].ext)))) 
        
        self.get_cur_volume_and_write_out(fname)
        

    def openVolume(self):
        cur_bfactor = self.bfactor_dial.value()
        cur_res = self.cutoff_dial.value()
        cur_res_str = 'd'.join(cur_res.__str__().split('.'))
        
        fname = 'recvol_{0}Bfac_{1}A.hdf'.format(abs(int(cur_bfactor)), cur_res_str)
        self.get_cur_volume_and_write_out(fname)
        self.external_launcher.qlaunch_open_file(os.path.abspath(fname))



# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to extract amplitudes and phases from desired layer lines of class averages
"""
from EMAN2 import Util, EMData, EMNumPy, periodogram
from collections import OrderedDict, namedtuple
from filter import filt_table
from functools import partial
import os
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.csgui import QTabWidgetCloseable, NumbersOptionsGuiWindow
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import DiagnosticPlot
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segclassexam import SegClassExam
from spring.segment2d.segmentalign2d import SegmentAlign2d
from spring.segment2d.segmentexam import SegmentExam
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.springgui.springdataexplore import SpringCommon, SpringDataExplore
import sys
from utilities import model_blank

from PyQt4.QtCore import Qt, SIGNAL
from PyQt4.QtGui import QApplication, QWidget, QComboBox, QStackedWidget, QSplitter, QFont, QToolTip, QGridLayout, \
    QDoubleSpinBox, QLabel
from matplotlib.backends.backend_pdf import PdfPages
from matplotlib.font_manager import FontProperties
from scipy import interpolate
from tabulate import tabulate

import numpy as np


class SegClassLayerPar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'segclasslayer'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.segclasslayer_features = Features()
        self.feature_set = self.segclasslayer_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    

    def define_parameters_and_their_properties(self):
        self.feature_set = self.segclasslayer_features.set_class_avg_stack(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_interactive_vs_batch_mode(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_output_plot(self.feature_set, self.progname + '_diag.pdf',
                                                                       'Batch mode')

        self.feature_set = self.set_class_format_choice(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_pixelsize(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_exact_helix_width(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_bfactor_on_images(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_power_cutoff(self.feature_set)
        self.feature_set = self.segclasslayer_features.set_class_number_range_to_be_analyzed(self.feature_set)

        self.feature_set = self.set_layer_line_position(self.feature_set)
        self.feature_set = self.set_pad_option(self.feature_set)

    def define_program_states(self):
        self.feature_set.program_states['extract_layerlines']='Extracts layer lines from specified position'
        self.feature_set.program_states['visualize_layerlines']='Visualization of layer lines'


    def set_class_format_choice(self, feature_set):
        inp7 = 'Class format'
        feature_set.parameters[inp7] = str('real')
        feature_set.hints[inp7] = 'Choose whether class average stack is in real or power spectrum format.'
        
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['real', 'power'],
        'QComboBox')
        
        feature_set.level[inp7]='beginner'
        
        return feature_set


    def set_layer_line_position(self, feature_set):
        inp8 = 'Layer line positions'
        feature_set.parameters[inp8] = str('0.1234,0.234')
        feature_set.hints[inp8] = 'List of comma-separated values in reciprocal Angstrom'
        feature_set.properties[inp8] = feature_set.file_properties(1, ['*'], None)
        feature_set.level[inp8]='expert'
        feature_set.relatives[inp8]='Batch mode'
        
        return feature_set
    

    def set_pad_option(self, feature_set):
        inp9 = 'Pad option'
        feature_set.parameters[inp9] = bool(True)
        feature_set.hints[inp9] = 'If layer lines are part of a continuous lattice set, average will be padded so ' + \
        'that layer lines lie exactly on pixel grid'
        feature_set.level[inp9]='expert'
        feature_set.relatives[inp9]='Batch mode'
        
        return feature_set
        
    
class SegClassLayerGui(QWidget):

    def __init__(self, feature_set, parent = None):
        QWidget.__init__(self, parent)
        
        self.feature_set = feature_set
        self.properties = feature_set.properties
        
        self = SpringCommon().setup_spring_page_top(self, feature_set)
        
        self.segclasslayer = SegClassLayer(self.feature_set)
        
        start_cls, end_cls = self.segclasslayer.classno_range
        self.classes = ['{0} - {1:03}'.format(os.path.basename(self.segclasslayer.infile), each_class) \
                        for each_class in list(range(start_cls, end_cls + 1))]
    
        self.stackedComboBox = QComboBox()
        self.stackedComboBox.addItems(self.classes)
        self.layout.addWidget(self.stackedComboBox, 0, 3, 1, 1)
        
        self.angstrom_str = NumbersOptionsGuiWindow().convert_angstrom_string('Angstrom')
        
        self.stackedWidget = QStackedWidget()
        self.class_plot = DiagnosticPlot()

        container = self.segclasslayer.make_named_tuple_amp_phase_images()
        self.class_img_canvas = [None for each_class in list(range(start_cls, end_cls + 1))]
        self.class_ps_canvas = [None for each_class in list(range(start_cls, end_cls + 1))]
        self.class_data = [container(each_class, None, None, None) \
                           for each_id, each_class in enumerate(list(range(start_cls, end_cls + 1)))]
        self.bfactor_dials = [None for each_class in list(range(start_cls, end_cls + 1))]

        self.tabwidgets = []
        for each_id, each_class in enumerate(list(range(start_cls, end_cls + 1))):
            self.splitter = QSplitter(Qt.Horizontal)
            self.splitter_vert = QSplitter(Qt.Vertical)
            
            self.setCurrentDisplay(each_id)
            
            self.splitter_vert.addWidget(self.class_img_canvas[each_id])
            self.splitter_vert.addWidget(self.class_ps_canvas[each_id])
            self.splitter.addWidget(self.splitter_vert)
            
            self.class_ps_canvas[each_id].main_frame.setToolTip('Mouse button to display amplitude and phases of ' + \
            'corresponding layer line.')
            
            self.class_ps_canvas[each_id].picked_left_point.connect(partial(self.add_new_tab_with_layer_line_profile))
            self.class_ps_canvas[each_id].picked_middle_point.connect(partial(self.add_new_tab_with_layer_line_profile))
            
            fig = self.prepare_layer_line_profile_plot(self.class_plot, self.layer_profile, self.max_amp)
            self.layer_fig = SpringDataExplore(fig)    
            
            self.layer_plane = QWidget()
            self.layer_canvas = QGridLayout()
            self.layer_canvas.addWidget(self.layer_fig, 0, 0, 0, 4)
#             
            self.bfactor_label = QLabel()
            self.bfactor_label.setText('B-factor')
            self.layer_canvas.addWidget(self.bfactor_label, 1, 1, 1, 1)

            self.bfactor_dials[each_id] = QDoubleSpinBox()
            bfact_cutoff_tip = 'Apply a negative B-factor to improve visualization of dampened high resolution '  + \
            'layer lines.'
            self.bfactor_dials[each_id].setToolTip(bfact_cutoff_tip)
            self.bfactor_dials[each_id].setRange(-90000, 90000)
            self.bfactor_dials[each_id].setSingleStep(50)
            self.bfactor_dials[each_id].setDecimals(0)
            self.bfactor_dials[each_id].setValue(self.segclasslayer.bfactor)
            self.connect(self.bfactor_dials[each_id], SIGNAL('editingFinished()'), self.activateOrInactivateBfactor)
            self.layer_canvas.addWidget(self.bfactor_dials[each_id], 1, 2, 1, 1)
            self.layer_plane.setLayout(self.layer_canvas)
            
            self.tabWidget = QTabWidgetCloseable()
            self.tabWidget.addTab(self.layer_plane, 'Equator')
            self.tabwidgets.append(self.tabWidget)
            self.splitter.addWidget(self.tabWidget)
            self.stackedWidget.addWidget(self.splitter)
        
        self.connect(self.stackedComboBox, SIGNAL('currentIndexChanged(int)'), self.stackedWidget.setCurrentIndex)
        self.layout.addWidget(self.stackedWidget, 2, 0, 2, 5)
        
        self.setLayout(self.layout)
        
        self.setMouseTracking(True)
 

    def enterEvent(self, event):
        QToolTip.setFont(QFont('Courier', 8))
        

    def setCurrentDisplay(self, each_index):
        class_id = self.class_data[each_index].class_id

        class_img_pd, cls_img_np, cls_ps_np = self.segclasslayer.prepare_class_img_and_power_spectrum(class_id,
        self.class_plot)
        
        img_dim_A = cls_img_np.shape[0] * self.segclasslayer.pixelsize

        A_grid = np.array([(-img_dim_A, -img_dim_A), (-img_dim_A, img_dim_A), (img_dim_A, img_dim_A),
                               (img_dim_A, -img_dim_A)]).reshape((2,2,2))
        
        if self.class_img_canvas[each_index] == None:
            self.class_img_canvas[each_index] = SpringDataExplore()
        self.class_img_canvas[each_index].on_draw(cls_img_np, A_grid, [''] + ['Angstrom']*2, '2d', color_map='gray')

        nyquist = 1 / (2 * self.segclasslayer.pixelsize)

        recip_A_grid = np.array([(-nyquist, -nyquist), (-nyquist, nyquist), (nyquist, nyquist), 
                                 (nyquist, -nyquist)]).reshape((2,2,2))
        
        if self.class_ps_canvas[each_index] == None:
            self.class_ps_canvas[each_index] = SpringDataExplore()
        self.class_ps_canvas[each_index].on_draw(cls_ps_np, recip_A_grid, [''] + ['1/Angstrom']*2, '2d', color_map='hot')

        amp_img, phase_img = self.segclasslayer.prepare_amplitude_and_phase_image(class_img_pd)
        self.layer_profile = self.segclasslayer.get_amplitude_and_phase_at_position(amp_img, phase_img, 1)
        
        self.fourier_dim = amp_img.get_ysize() / 2

        container = self.segclasslayer.make_named_tuple_amp_phase_images()
        self.max_amp = 1.1 * max(np.append(self.layer_profile.left_amp, self.layer_profile.right_amp))
        self.class_data[each_index]=(container(class_id, amp_img, phase_img, self.max_amp))

            
    def activateOrInactivateBfactor(self):
        cur_id = self.stackedComboBox.currentIndex()
        self.segclasslayer.bfactor = self.bfactor_dials[cur_id].value()
        self.setCurrentDisplay(cur_id)


    def add_new_tab_with_layer_line_profile(self, index_pair):
        y_pos = index_pair[1]
        fourier_pixel = int(abs(y_pos - self.fourier_dim)) + 1

        nyquist = 1 / (2 * self.segclasslayer.pixelsize)
        fourier_pix_A = fourier_pixel * nyquist / self.fourier_dim 
            
        cur_id = self.stackedComboBox.currentIndex()
        amp_img = self.class_data[cur_id].amp
        phase_img = self.class_data[cur_id].phase
        max_amp = self.class_data[cur_id].max_amp
        
        layer_profile = self.segclasslayer.get_amplitude_and_phase_at_position(amp_img, phase_img, fourier_pixel)
        
        fig = self.prepare_layer_line_profile_plot(self.class_plot, layer_profile, max_amp)
        
        self.layer_canvas = SpringDataExplore(fig)    

        besselorder, primarymax, primarymaxwidth = self.segclasslayer.get_bessel_table_quant(self.segclasslayer.helixwidth)

        tool_str = tabulate([['{0}'.format('Bessel order')] + besselorder, 
                             ['{0}'.format('Primary max at 2*Pi*r*R')] + primarymax, 
                             ['{0}'.format('At helix radius (R={0} A)'.format(self.segclasslayer.helixwidth / 2))] + primarymaxwidth],
                            tablefmt='grid')
        
        self.layer_canvas.main_frame.setToolTip(tool_str)

        title = '{0:.04} 1/{1}'.format(fourier_pix_A, self.angstrom_str)
        self.tabwidgets[cur_id].addTab(self.layer_canvas, title)
        self.tabwidgets[cur_id].setCurrentIndex(self.tabwidgets[cur_id].indexOf(self.layer_canvas))
        
        
    def prepare_layer_line_profile_plot(self, class_plot, layer_profile, ampmax=None):
        arrresolution = SegmentExam().make_oneoverres(layer_profile.left_phase, self.segclasslayer.pixelsize)
        fig = class_plot.create_next_figure()
        ax1 = fig.add_subplot(111)
        
        ax1, ax2 = self.segclasslayer.add_amplitude_and_phase_difference_to_plot(layer_profile,
            arrresolution, ax1, ampmax)
        
        self.segclasslayer.add_labels_to_plot(ax1, ax2)
        
        return fig

        
class SegClassLayerExtract(object):
    """
    * Class that holds functions for extracting layer lines including amplitude and phase from class averages

    * __init__ Function to interpret multi-input parameters

    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.infilestack = p['Class average stack']
            self.infile = self.infilestack
            self.batch_mode = p['Batch mode']
            self.outfile = p['Diagnostic plot']
            self.class_format = p['Class format']
            self.pixelsize = p['Pixel size in Angstrom']
            self.helixwidth = p['Precise helix width in Angstrom']
            self.helixwidthpix = int(round(self.helixwidth/self.pixelsize))
            self.bfactor = p['B-Factor']
            self.rescutoff = p['Power spectrum resolution cutoff in 1/Angstrom']
            self.classno_range = p['Class number range to be analyzed']
            self.layerlines = p['Layer line positions'].split(',')
            self.padoption = p['Pad option']

            self.binfactor = int(round(1/(self.rescutoff*self.pixelsize*2)))

            self.stack = EMData()
            self.stack.read_image(self.infilestack, 0)
            self.segsizepix = self.stack.get_xsize()

            if self.binfactor > 1:
                self.infilestack, self.segsizepix, self.helixwidthpix, self.pixelsize = \
                SegmentExam().apply_binfactor(self.binfactor, self.infilestack, self.segsizepix, self.helixwidthpix,
                self.pixelsize)


    def compute_phase_difference(self, leftph, rightph):
        """
        >>> from spring.segment2d.segclasslayer import SegClassLayer
        >>> s = SegClassLayer()
        >>> s.compute_phase_difference(np.array([50] * 2), np.array([-120] * 2))
        array([170, 170])
        >>> s.compute_phase_difference(np.array([170]), np.array([180]))
        array([-10])
        >>> s.compute_phase_difference(np.array([-70]), np.array([170]))
        array([120])
        """
        phase_diff_plus360 = leftph + 360 - rightph
        phase_diff = leftph - rightph
        phase_diff_minus360 = leftph - rightph - 360
            
        dist = np.append(phase_diff, [phase_diff_minus360, phase_diff_plus360])
        
        dist = dist.reshape((3, phase_diff.size))
        min_args = [np.argmin(np.abs(each_dist)) for each_dist in dist.T] 
            
        dist_min = np.array([each_dist[min_args[each_min_id]] for each_min_id, each_dist in enumerate(dist.T)])
        
        dist_min = dist_min.reshape((phase_diff.shape))

        return dist_min
    

    def compute_phase_difference_old(self, leftph, rightph):
        """
        >>> from spring.segment2d.segclasslayer import SegClassLayer
        >>> s = SegClassLayer()
        >>> s.compute_phase_difference_old(np.array([50]), np.array([-120]))
        array([ 170.])
        >>> s.compute_phase_difference_old(np.array([170]), np.array([180]))
        array([-10.])
        >>> s.compute_phase_difference_old(np.array([-70]), np.array([170]))
        array([ 120.])
        """
        phasediffarr = np.array([])
        
        for each_left, each_right in zip(leftph, rightph):
            phase_diff_plus360 = each_left + 360 - each_right
            phase_diff = each_left - each_right
            phase_diff_minus360 = each_left - each_right - 360
            
            dist = np.array([phase_diff_plus360, phase_diff, phase_diff_minus360])
            
            phasediffarr = np.append(phasediffarr, dist[np.argmin(np.abs(dist))])
            
        return phasediffarr
    
    
    def make_named_tuple_layer_line(self):
        return namedtuple('layer_line', 'left_amp right_amp left_phase right_phase')
    
    
    def center_leftside(self, leftarr, rightarr):
        """
        * Function to flip and center left image side with respect to right image side and complement first pixel

        #. Input: left-side amplitude, right-side amplitude
        #. Output: complemented and mirrored left-side amplitudes
        #. Usage: leftarrflip = center_leftside(leftarr, rightarr)
        """
        # flip left side
        leftarrflip = np.flipud(leftarr)

        leftarrflip = np.delete(leftarrflip, -1)
        leftarrflip = np.insert(leftarrflip, 0, rightarr[0])

        return leftarrflip
    

    def retrieve_ampnphases(self, llampprofile, llphprofile):
        """
        * Function to retrieve amplitude and phases along a layer line of interest

        #. Input: layerline amplitude profile, layerline phase profile
        #. Output: leftamplitude, rightamplitude and phase difference 
        #. Usage: leftamp, rightamp, phasediff = retrieve_ampnphases(llampprofile, llphprofile)
        """

        llampprofilearr = np.copy(EMNumPy.em2numpy(llampprofile))
        llphprofilearr = np.copy(EMNumPy.em2numpy(llphprofile))
        # split in half
        leftamp, rightamp = np.split(llampprofilearr, 2)
        leftph, rightph = np.split(llphprofilearr, 2)

        leftamp = self.center_leftside(leftamp, rightamp)
        leftph = self.center_leftside(leftph, rightph)

        layer = self.make_named_tuple_layer_line()
        
        return layer(leftamp, rightamp, np.rad2deg(leftph), np.rad2deg(rightph))
    

    def prepare_amplitude_and_phase_image(self, avg):
        if self.class_format == 'real':
            fftavg = avg.do_fft()
            amplitude_of_avg = fftavg.get_fft_amplitude()
            phase_of_average = fftavg.get_fft_phase()
        elif self.class_format == 'power':
            avg_np = np.copy(EMNumPy.em2numpy(avg))
            avg_np = np.roll(avg_np, avg.get_xsize() / 2, 0)
            amplitude_of_avg = EMNumPy.numpy2em(np.copy(avg_np))
            phase_of_average = model_blank(avg.get_xsize(), avg.get_ysize(), 1)

        return amplitude_of_avg, phase_of_average


    def get_amplitude_and_phase_at_position(self, amplitude_of_avg, phase_of_average, lineimgpos):
        llampprofile = amplitude_of_avg.get_row(lineimgpos)
        llphprofile = phase_of_average.get_row(lineimgpos)
        layer = self.retrieve_ampnphases(llampprofile, llphprofile)
        
        return layer
    

    def extract_layer_line_amplitude_and_phases(self, avg, layerlinespix):
        amplitude_of_avg, phase_of_average = self.prepare_amplitude_and_phase_image(avg)
        
        layer_profiles = []
        for self.line in layerlinespix:
            # compute pixel position
            lineimgpos = int(self.line)
            
            layer = self.get_amplitude_and_phase_at_position(amplitude_of_avg,
            phase_of_average, lineimgpos)
            
            layer_profiles.append(layer)
            
        return layer_profiles
            

    def pad_avg(self, layerlines, classno):
        """
        * Function to pad average to fit layer lines directly onto pixel row (pads classavg and adjusts segmentsize)

        #. Input: list of layerlines in pixel
        #. Output: list of adjusted layerlines in pixel
        #. Usage: layerlines = pad_avg(layerlines)
        """

        layerorder = np.arange(len(layerlines))
        slope, icept = np.polyfit(layerorder, layerlines, 1)

        if 100 < self.segsizepix < 150:
            uproundedslope = round(slope + 5)
        elif self.segsizepix < 100:
            uproundedslope = round(slope + 10)
        else:
            uproundedslope = round(slope + 1)

        pdsegsizepix = int(round(self.segsizepix*uproundedslope)/slope)

        if (pdsegsizepix)%2 != 0:
            pdsegsizepix = pdsegsizepix + 1

        img = Util.pad(self.avg, pdsegsizepix, pdsegsizepix, 1, 0, 0, 0, '0' )
        newlayerlines = [uproundedslope*(layerorder + 1) for layerorder, line in enumerate(layerlines)]

        self.avg = img
        layerlinespix = newlayerlines
        self.segsizepix = pdsegsizepix

        self.log.ilog('Class {classno} will be padded to {xsize} x {ysize} pixels'.format(classno=classno,
        xsize=pdsegsizepix, ysize=pdsegsizepix))
        
        self.log.ilog('Layer lines are plotted from Fourier pixels:{0}'.format(layerlinespix))

        return layerlinespix
        
        
class SegClassLayerGuiSupport(SegClassLayerExtract):
    def make_named_tuple_amp_phase_images(self):
        return namedtuple('image', 'class_id amp phase max_amp')
    
    def get_class_img(self, class_id):
        class_img = EMData()
        class_img.read_image(self.infilestack, class_id)
        segment_size = class_img.get_xsize()
        if self.bfactor != 0:
            filter_coefficients = SegmentAlign2d().prepare_filter_function(False, 1 / 300.0, True, 0.02,
            self.pixelsize, segment_size, 0.08, False, None, self.bfactor) 

            class_img = filt_table(class_img, filter_coefficients)

        return segment_size, class_img


    def prepare_class_img_and_power_spectrum(self, class_id, class_plot):
        segment_size, class_img = self.get_class_img(class_id)
        cls_img_np = np.copy(EMNumPy.em2numpy(class_img))

        if self.class_format == 'real':
            helixmask = SegmentExam().make_smooth_rectangular_mask(self.helixwidthpix, segment_size * 0.8,
            segment_size, 0.15)
            
            padsize = 4
            class_img_pd = Util.pad(class_img * helixmask, padsize * segment_size, padsize * segment_size, 1, 0, 0, 0, '0')
            
            cls_ps = periodogram(class_img_pd)
            cls_ps.process_inplace('normalize')
            cls_ps_np = np.copy(EMNumPy.em2numpy(cls_ps))
        elif self.class_format == 'power':
            cls_ps_np = np.copy(cls_img_np)
        
        return class_img_pd, cls_img_np, cls_ps_np 
    

class SegClassLayer(SegClassLayerGuiSupport):
    def add_amplitude_and_phase_difference_to_plot(self, layer_profile, arrresolution, ax1, ampmax=None):
        ax1.plot(arrresolution, layer_profile.left_amp, label='Left quadrant')
        ax1.plot(arrresolution, layer_profile.right_amp, label='Right quadrant amplitudes')
        if ampmax is not None:
            ax1.set_ylim(0, ampmax)
        ax1.set_xlim(arrresolution[0], arrresolution[-1])
        ax1.set_yticks([])
        
        ax2 = ax1.twinx()
        ax2.set_xlim(arrresolution[0], arrresolution[-1])
        ax2.set_ylim(-190, 230)
        
        phase_diff = self.compute_phase_difference(layer_profile.left_phase, layer_profile.right_phase)
        ax2.plot(arrresolution, phase_diff, 'r.', label='Phase difference left/right')
        
        ax2.fill([0,arrresolution[-1], arrresolution[-1], 0], [-90,-90, 90, 90], 'grey', alpha=0.05, 
        label='Even Bessel order')
#        ax2.plot(arrresolution, layer_profile.left_phase, '.', label='phase left')
#        ax2.plot(arrresolution, layer_profile.right_phase, 'x', label='phase right')
        ax2.set_yticks([-180, -90, 0, 90, 180])
        [t.set_fontsize(6) for t in ax1.get_xticklabels() + ax1.get_yticklabels() + ax2.get_yticklabels()]
        
        return ax1, ax2
    

    def determine_first_maxima_computationally(self, layer_profile, ax1):
        finestep = (self.arrresolution[1] - self.arrresolution[0]) / 100
        resarrpol = np.arange(self.arrresolution[0], self.arrresolution[-1], finestep)
        
        t = interpolate.splrep(self.arrresolution, layer_profile.left_amp, k=3, s=0)
        leftamppol = interpolate.splev(resarrpol, t)
        ind, val = SegmentExam().find_local_extrema(leftamppol)
        leftmax = resarrpol[ind[-1]]
        leftmaxval = val[-1]
        ax1.text(leftmax, 0.2 * leftmaxval, 'Max {0:.4}'.format(leftmax), fontsize=6)
        
        t = interpolate.splrep(self.arrresolution, layer_profile.right_amp, k=3, s=0)
        rightamppol = interpolate.splev(resarrpol, t)
        ind, val = SegmentExam().find_local_extrema(rightamppol)
        rightmax = resarrpol[ind[-1]]
        ax1.text(rightmax, 0.4 * leftmaxval, 'Max {0:.4}'.format(rightmax), fontsize=6)
        

    def add_labels_to_plot(self, ax1, ax2):
        ax1.set_xlabel('Resolution in 1/Angstrom', fontsize=8)
        ax1.set_ylabel('LL amplitudes', fontsize=8)
        ax1.legend(loc='upper left', ncol=1, prop=FontProperties(size='x-small'))
        
        ax2.set_ylabel('LL phase difference (degrees)', fontsize=8)
        ax2.legend(loc='upper right', ncol=1, prop=FontProperties(size='x-small'))
        
        return ax1, ax2
    

    def plot_individual_layer_lines(self, diag_plot, layerlines, layer_profiles):
        plots = OrderedDict()
        for self.order, line in enumerate(layerlines):
            llplot = 'plot{0}'.format(self.order)
            plots[llplot] = diag_plot.plt.subplot2grid((len(layerlines) + 1, 1), (self.order, 0), colspan=1, rowspan=1)
            # plot figure
            plots[llplot].set_title('{order}. layer line ({res} 1/Angstrom)'.format(order=self.order, res=line),
            fontsize=9)
            
            llplottwin = 'plot{0}twin'.format(self.order)
            plots[llplot], plots[llplottwin] = self.add_amplitude_and_phase_difference_to_plot(layer_profiles[self.order],
            self.arrresolution, plots[llplot])
            
            self.determine_first_maxima_computationally(layer_profiles[self.order], plots[llplot])
            
            plots[llplottwin].minorticks_on()
            plots[llplottwin].grid(True)
            self.log.ilog('Layer line {order} was visualized left and right quadrant amplitudes and phase ' + \
            'difference'.format(order=self.order))
            # last plot
            if self.order == (len(layerlines) - 1):
                self.add_labels_to_plot(plots[llplot], plots[llplottwin])
                
                tbl = diag_plot.plt.subplot2grid((len(self.layerlines) + 1,1), (self.order + 1,0), colspan=1, rowspan=1,
                frameon=False)
        
                self.add_besseltable(diag_plot, tbl)
                self.log.ilog('Final Bessel order lookup table added')
                
        return diag_plot
    

    def determine_max(self):
        """
        * Function to determine maximum of layer line
        """
        ind1 = np.lexsort((self.arrresolution, self.leftamp[self.order]))
        ind2 = np.lexsort((self.arrresolution, self.rightamp[self.order]))
        if self.leftamp[self.order][ind1[-1]] > self.rightamp[self.order][ind2[-1]]:
            self.maxx = self.arrresolution[ind1[-1]]
            self.maxy = self.leftamp[self.order][ind1[-1]]
        else:
            self.maxx = self.arrresolution[ind2[-1]]
            self.maxy = self.rightamp[self.order][ind2[-1]]

        return self.maxx, self.maxy


    def get_bessel_table_quant(self, helixwidth):
        """
        >>> from spring.segment2d.segclasslayer import SegClassLayer
        >>> s = SegClassLayer()
        >>> s.get_bessel_table_quant(100) #doctest: +NORMALIZE_WHITESPACE
        ([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19], 
        [0.0, 1.8, 3.1, 4.2, 5.3, 6.4, 7.5, 8.6, 9.6, 10.7, 11.8, 12.8, 13.9, 
        14.9, 16.0, 17.0, 18.1, 19.1, 20.1, 21.2], [0.0, 0.006, 0.01, 0.013, 
        0.017, 0.02, 0.024, 0.027, 0.031, 0.034, 0.038, 0.041, 0.044, 0.047, 
        0.051, 0.054, 0.058, 0.061, 0.064, 0.067])
        """
        primarymax = SegClassReconstruct().get_list_of_bessel_order_maxima(20)
        primarymax = np.around(primarymax, decimals=1)

        primarymaxwidth = primarymax / (2 * np.pi * helixwidth / 2.0)
        primarymaxwidth = np.around(primarymaxwidth, decimals=3)

        besselorder = np.arange(len(primarymax), dtype=int)

        return besselorder.tolist(), primarymax.tolist(), primarymaxwidth.tolist()


    def add_besseltable(self, diag_plot, tbl, helixwidth=None):
        """
        * Function to add Bessel order look-up table to printout

        #. Input: helix width in Angstrom
        #. Output: table with Bessel order and their corroesponding primary maximum and expected \
            maximum for helix width
        #. Usage: add_besseltable(helixwidth)
        """
        if helixwidth is None: helixwidth = self.helixwidth

        besselorder, primarymax, primarymaxwidth = self.get_bessel_table_quant(helixwidth)

        tbl.set_title('Bessel order look-up table (from Stewart 1988)', fontsize=10)
        tbl.xaxis.set_visible(False)
        tbl.yaxis.set_visible(False)
        
        rowLabels = ['Bessel order', 
        'Primary maximum at 2*Pi*r*R', 'at helix radius \nof R={width} Angstrom'.format(width=helixwidth/2)]
        
        cellText = np.row_stack((besselorder, primarymax, primarymaxwidth))

        the_table = diag_plot.plt.table(cellText=cellText, rowLabels=rowLabels, loc='center')
        the_table.auto_set_font_size(False)
        the_table.set_fontsize(4)


    def extract_layerlines(self, infilestack, classno):
        self.log.fcttolog()

        self.avg = EMData()
        self.avg.read_image(infilestack, classno)

        layerlines = np.array([float(line.strip()) for line in self.layerlines])
        layerlinespix = np.round(self.pixelsize*self.segsizepix*layerlines)

        if self.padoption is True:
            layerlinespix = self.pad_avg(layerlinespix, classno)

        layer_profiles = self.extract_layer_line_amplitude_and_phases(self.avg, layerlinespix)

        return layerlines, layer_profiles
        
        
    def visualize_layerlines(self, figno, layerlines, layer_profiles):
        """
        * Function to visualize layer lines

        #. Input: figurenumber, list of layerlines (1/Angstrom), list of left-side centered amplitude \
            arrays, list of right-side centered amplitude arrays, list of phasedifference arrays
        #. Output: figure with stacked amplitude profile and phase difference plots
        #. Usage: figure = visualize_layerlines(figno, layerlines, leftamp, rightamp, phasediff)
        """
        self.log.fcttolog()
        layerline_plot = DiagnosticPlot()
        layerline_plot.plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=0.65)
        
#        self.fig = layerline_plot.add_header_and_footer(self.feature_set)

        self.arrresolution = SegmentExam().make_oneoverres(layer_profiles[0].left_phase, self.pixelsize)
 
        layerline_plot = self.plot_individual_layer_lines(layerline_plot, layerlines, layer_profiles)

        return layerline_plot.fig


    def print_layerlines_as_requested(self):
        if os.path.splitext(self.outfile)[-1].endswith('pdf'):
            self.pdf = PdfPages(self.outfile)
        self.classno_range = SegClassExam().check_maximum_class_number(self.infilestack, self.classno_range)
        classno_start, classno_end = self.classno_range
        self.log.plog(10)
        
        classes_iter = list(range(classno_start, classno_end + 1))
        for each_class_index in classes_iter:
            if len(classes_iter) > 1:
                plot_file, self.feature_set = SegClassExam().rename_plot_title_for_multiple_classes(self.infile, 
                self.outfile, classno_start, classno_end, each_class_index, self.feature_set)
            else:
                plot_file = self.outfile
            
            layerlines, layer_profiles = self.extract_layerlines(self.infilestack, each_class_index)
            self.fig = self.visualize_layerlines(each_class_index, layerlines, layer_profiles)
            
            if os.path.splitext(self.outfile)[-1].endswith('pdf'):
                self.pdf.savefig(self.fig)
            elif not os.path.splitext(self.outfile)[-1].endswith('pdf') and classno_end != classno_start:
                self.fig.savefig(plot_file, dpi=600)
            else:
                self.fig.savefig(self.outfile, dpi=600)
            self.log.plog(100 * (each_class_index + 1) / (classno_end + 1))
        
        if os.path.splitext(self.outfile)[-1].endswith('pdf'):
            self.pdf.close()


    def launch_segclasslayer_gui(self, feature_set):
        self.log.fcttolog()
        app = QApplication(sys.argv)
        gridexplor = SegClassLayerGui(feature_set)
        gridexplor.show()
        app.exec_()
        

    def extract_and_visualize_layer_lines(self):
        if self.batch_mode:
            self.print_layerlines_as_requested()
        else:
            self.launch_segclasslayer_gui(self.feature_set)
            
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = SegClassLayerPar()
    mergeparset = OptHandler(parset)

    ######## Program
    stack = SegClassLayer(mergeparset)
    stack.extract_and_visualize_layer_lines()

if __name__ == '__main__':
    main()
